#File: functionGen.tcl
#Syscomp Electronic Design Ltd.
#www.syscompdesign.com
#JG
#Copyright 2009 Syscomp Electronic Design

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA
package provide functionGen 1.0
package require Img

namespace eval wave {

#---=== Waveform Global Variables ===---
set minFrequencyLimit 0.1
set maxFrequencyLimit 10000000
set minFrequency 0.1
set maxFrequency 3000000
set sliderRange 425
set ampSliderLength 470
set offSliderLength 470
set ddsResolution 0.031247735
set defaultFrequency 1000.0
set frequencyDisplay $defaultFrequency
set outputEnabled 1
set displaySize 150
set sweepEnabled 0
set sweepMode linear
set sweepLinearStep 1
set sweepLogarithmicStep 1.1
set sweepTime 1000
set sweepHandle null
set sweepDirection up
set sweepInitialFrequency $minFrequency
set sweepBehavior cycle
set sweepTriggerMode manual
set sweepTrigOutEnable 0

#Waveform Images:
set freqImage [image create photo -file "$images/freqImage.gif"]
set ampImage [image create photo -file "$images/ampImage.gif"]
set offImage [image create photo -file "$images/offImage.gif"]

#Arbitrary Waveforms:
set customWaveform {}

#---=== Export Public Procedures ===---
namespace export buildFunctionGen
namespace export setWavePath
namespace export getWavePath
}

#---=== Procedures ===---

#Set Waveform Path
#
proc ::wave::setWavePath {wavePath} {
	variable wave
	
	set wave(path) $wavePath
}

proc ::wave::getWavePath {} {
	variable wave
	
	return $wave(path)
}


proc ::wave::buildFunctionGen {} {


	set wavePath [wave::getWavePath]
	
	#Frame for Waveform Generator Controls
	frame $wavePath	\
		-relief raised	\
		-borderwidth 2

	#Waveform frequency controls
	frame $wavePath.freq	\
		-relief groove	\
		-borderwidth 2
		
	label $wavePath.title	\
		-text "Waveform Generator"	\
		-font {-weight bold -size -16}
		
	scale $wavePath.freq.freqSlider	\
		-from $::wave::sliderRange	\
		-to 1		\
		-variable ::wave::frequencyPosition	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 1	\
		-showvalue 0	\
		-length $::wave::sliderRange	\
		-command ::wave::adjustFrequency
		
	label $wavePath.freq.title	\
		-image $::wave::freqImage

	button $wavePath.freq.topValue	\
		-textvariable ::wave::maxFrequency	\
		-width 8	\
		-command {::wave::setMaxFrequency}
		
	button $wavePath.freq.bottomValue	\
		-textvariable ::wave::minFrequency	\
		-width 8	\
		-command {::wave::setMinFrequency}

	grid $wavePath.freq.title -row 0 -column 0 -sticky n
	grid $wavePath.freq.topValue -row 1 -column 0
	grid $wavePath.freq.freqSlider -row 2 -column 0
	grid $wavePath.freq.bottomValue -row 3 -column 0
	
	#Waveform Amplitude Controls
	frame $wavePath.amp	\
		-relief groove	\
		-borderwidth 2
		
	scale $wavePath.amp.ampSlider	\
		-from 100		\
		-to 0			\
		-variable ::wave::amplitude	\
		-orient vertical	\
		-showvalue 1	\
		-length $wave::ampSliderLength	\
		-tickinterval 5	\
		-resolution 1	\
		-command ::wave::adjustAmplitude
		
	label $wavePath.amp.title	\
		-image $::wave::ampImage

	grid $wavePath.amp.title -row 0 -column 0 -sticky n
	grid $wavePath.amp.ampSlider -row 1 -column 0 -sticky n
	
	#Waveform Amplitude Controls
	frame $wavePath.off	\
		-relief groove	\
		-borderwidth 2
		
	scale $wavePath.off.offSlider	\
		-from 10		\
		-to -10			\
		-variable ::wave::offset	\
		-orient vertical	\
		-showvalue 1	\
		-length $wave::offSliderLength	\
		-tickinterval 1	\
		-resolution 0.1	\
		-command ::wave::adjustOffset
		
	label $wavePath.off.title	\
		-text "Offset"	\
		-image $::wave::offImage

	grid $wavePath.off.title -row 0 -column 0 -sticky n
	grid $wavePath.off.offSlider -row 1 -column 0 -sticky n

	#Waveform selection controls
	frame $wavePath.wave	\
		-relief groove	\
		-borderwidth 2

	canvas $wavePath.wave.waveDisplay	\
		-width $wave::displaySize	\
		-height $wave::displaySize	\
		-background white	\
		-borderwidth 2
		
	$wavePath.wave.waveDisplay create line	\
		0 [expr {$wave::displaySize/2.0}]	\
		$wave::displaySize [expr {$wave::displaySize/2.0}]	\
		-fill black	\
		-tag displayGrid
	$wavePath.wave.waveDisplay create rectangle	\
		4 4	\
		$wave::displaySize $wave::displaySize	\
		-outline black	\
		-tag displayGrid
		
	button $wavePath.wave.freqDisplay	\
		-relief sunken	\
		-borderwidth 3	\
		-textvariable ::wave::frequencyDisplay	\
		-font {-weight bold -size -14}	\
		-width 10	\
		-background black	\
		-foreground red	\
		-command {::wave::setFrequency}
		
	button $wavePath.wave.sine	\
		-text "Sine"	\
		-command {sendCommand "W 0"; sendCommand "n"; wave::updateDisplay 0}
		
	button $wavePath.wave.square	\
		-text "Square"	\
		-command {sendCommand "W 2"; sendCommand "n"; wave::updateDisplay 2}
		
	button $wavePath.wave.triangle	\
		-text "Triangle"	\
		-command {sendCommand "W 1"; sendCommand "n"; wave::updateDisplay 1}
		
	button $wavePath.wave.sawtooth	\
		-text "Sawtooth"	\
		-command {sendCommand "W 3"; sendCommand "n"; wave::updateDisplay 3}
		
	button $wavePath.wave.custom	\
		-text "Stored Custom"	\
		-command {sendCommand "W 4"; sendCommand "n"; wave::updateDisplay 4}
		
	button $wavePath.wave.noise	\
		-text "Noise"	\
		-command {::wave::enableNoise}
	
	button $wavePath.wave.loadCustom	\
		-text "Load Custom"	\
		-command {wave::loadCustom}
		
	button $wavePath.wave.saveCustom	\
		-text "Save Custom"	\
		-command {wave::saveCustom}

	grid $wavePath.wave.waveDisplay -row 0 -column 0 -columnspan 2
	grid $wavePath.wave.freqDisplay -row 1 -column 0 -columnspan 2
	grid $wavePath.wave.sine -row 2 -column 0 -sticky we
	grid $wavePath.wave.square -row 2 -column 1 -sticky we
	grid $wavePath.wave.triangle -row 4 -column 0 -sticky we
	grid $wavePath.wave.sawtooth -row 4 -column 1 -sticky we
	grid $wavePath.wave.custom -row 5 -column 0 -sticky we
	grid $wavePath.wave.noise -row 5 -column 1 -sticky we
	grid $wavePath.wave.loadCustom -row 6 -column 0 -sticky we
	grid $wavePath.wave.saveCustom -row 6 -column 1 -sticky we

	#Frequency Slider controls 
	frame $wavePath.control	\
		-relief groove	\
		-borderwidth 2
		
	label $wavePath.control.modeTitle	\
		-text "Frequency\nControl Mode:"	\
		-font {-weight bold -size -12}
		
	radiobutton $wavePath.control.logMode	\
		-text "Logarithmic"	\
		-value log	\
		-variable ::wave::sliderMode
		
	#Select Logarithmic Mode by Default
	$wavePath.control.logMode select
	
	radiobutton $wavePath.control.linMode	\
		-text "Linear"	\
		-value linear	\
		-variable ::wave::sliderMode
		
	grid $wavePath.control.modeTitle -row 0 -column 0 -sticky nw
	grid $wavePath.control.logMode -row 1 -column 0 -sticky nw
	grid $wavePath.control.linMode -row 2 -column 0 -sticky nw
	
	#Waveform Output Controls
	frame $wavePath.outputControl	\
		-relief groove	\
		-borderwidth 2
		
	label $wavePath.outputControl.title	\
		-text "Waveform Output:"	\
		-font {-weight bold -size -12}
	
	radiobutton $wavePath.outputControl.enabled	\
		-text "Enabled"	\
		-value 1	\
		-variable wave::outputEnabled	\
		-command wave::waveformOutput
	
	$wavePath.outputControl.enabled select
		
	radiobutton $wavePath.outputControl.disabled	\
		-text "Disabled"	\
		-value 0	\
		-variable wave::outputEnabled	\
		-command wave::waveformOutput
		
	grid $wavePath.outputControl.title -row 0 -column 0 -sticky nw
	grid $wavePath.outputControl.enabled -row 1 -column 0 -sticky nw
	grid $wavePath.outputControl.disabled -row 2 -column 0 -sticky nw
	
	#Automatic Sweep Controls
	frame $wavePath.sweep	\
		-relief groove	\
		-borderwidth 2
		
	label $wavePath.sweep.title	\
		-text "Frequency Sweep:"	\
		-font {-weight bold -size -12}
		
	radiobutton $wavePath.sweep.disable	\
		-text "Disabled"	\
		-value 0	\
		-variable wave::sweepEnabled	\
		-command wave::sweepControl
		
	radiobutton $wavePath.sweep.enable	\
		-text "Enabled"	\
		-value 1	\
		-variable wave::sweepEnabled	\
		-command wave::sweepControl
		
	button $wavePath.sweep.settings	\
		-text "Sweep Settings"	\
		-command wave::sweepSettings
		
	grid $wavePath.sweep.title -row 0 -sticky nw
	grid $wavePath.sweep.disable -row 1 -sticky nw
	grid $wavePath.sweep.enable -row 2 -sticky nw
	grid $wavePath.sweep.settings -row 3 -sticky nw

	#Waveform Generator Controls
	grid $wavePath.title -row 0 -column 0 -sticky w
	grid $wavePath.wave -row 1 -column 0 -sticky n
	grid $wavePath.control -row 2 -column 0 -sticky we
	grid $wavePath.outputControl -row 3 -column 0 -sticky we
	grid $wavePath.sweep -row 4 -column 0 -sticky we
	grid $wavePath.amp -row 1 -column 1 -sticky n -rowspan 5 -padx 5
	grid $wavePath.off -row 1 -column 2 -sticky n -rowspan 5 -padx 5
	grid $wavePath.freq -row 1 -column 3 -rowspan 5 -padx 5
	

}

# Adjust Waveform Generator Frequency
#--------------------------------------------
# This procedure is a service routine for the waveform generator frequency
# slider control.  It converts the slider position into a frequency, calls the
# procedure to update the hardware, and updates the frequency display.
proc ::wave::adjustFrequency {sliderArg} {
	variable minFrequency
	variable maxFrequency
	variable sliderRange
	variable frequencyDisplay
	variable sliderMode
	
	
	if {$sliderMode == "log"} {
		#Logarithmic interpretation of slider position
		set logMin [expr {log10($minFrequency)}]
		set logMax [expr {log10($maxFrequency)}]
		set b $logMin
		set m [expr {($logMax-$logMin)/($sliderRange-1)}]
		set y [expr {$m*($sliderArg-1)+$b}]
		set frequency [expr {pow(10,$y)}]
		
	} else {
		#Linear interpretation of slider position
		set b $minFrequency
		set m [expr {($maxFrequency-$minFrequency)/($sliderRange-1)}]
		set y [expr {$m*($sliderArg-1)+$b}]
		set frequency $y
	}
	
	#Round to the nearest tenth of a hertz
	set waveFrequency [format "%.1f" $frequency]
	
	#Update the hardware with the new frequency
	::wave::sendFrequency $waveFrequency
	
	#Update the frequency display
	set frequencyDisplay "$waveFrequency Hz"
	
}

# Send Frequency Setting to Hardware
#-------------------------------------------
# This procedure accepts a frequency value, converts it to a four byte number,
# and sends it to the hardware.
proc ::wave::sendFrequency {freq} {
	variable ddsResolution
	
	#Calculate the phase integer
	set freqOutput [expr {round($freq/$ddsResolution)}]

	#Break the phase integer into a four byte number
	set byte3 [expr {round(floor($freqOutput/pow(2,24)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,24))}]
	
	set byte2 [expr {round(floor($freqOutput/pow(2,16)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,16))}]
	
	set byte1 [expr {round(floor($freqOutput/pow(2,8)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,8))}]
	
	set byte0 $freqOutput
	
	sendCommand "F $byte3 $byte2 $byte1 $byte0"
	
}

# Adjust Waveform Amplitude
#--------------------------------
# This is a service procedure for the amplitude slider control.  It converts
# the slider argument into an amplitude value and sends it to the hardware.
proc ::wave::adjustAmplitude {sliderArg} {
	
	set amplitude [expr {round(255.0*($sliderArg/100.0))}]
	
	sendCommand "A $amplitude"
}

# Adjust Waveform Offset
#--------------------------------
# This is a service procedure for the amplitude slider control.  It converts
# the slider argument into an amplitude value and sends it to the hardware.
proc ::wave::adjustOffset {sliderArg} {
	
	set offset [expr {round((-1)*$wave::offset*127/10.0)+127}]
	
	sendCommand "O $offset"
}

# Manually Set Frequency
#----------------------------
# This procedure is called when the user wants to manually set the waveform generator
# output frequency.  It presents the user with a dialog box where they can enter
# the desired output frequency.
proc ::wave::setFrequency {} {
	variable minFrequencyLimit
	variable maxFrequencyLimit
	variable frequencyDisplay
	
	#Dialog box for user to enter the new frequency
	set newFreq [Dialog_Prompt newF "New Frequency:"]
	
	if {$newFreq == ""} {return}
	
	#Make sure that we got a valid frequency setting
	if { [string is double -strict $newFreq] } {
		if { $newFreq >= $minFrequencyLimit && $newFreq <= $maxFrequencyLimit} {
			sendFrequency $newFreq
			set frequencyDisplay [format "%.1f" $newFreq]
			set frequencyDisplay "$frequencyDisplay Hz"
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency out of range"	\
			-type ok			\
			-icon warning
		}
	} else {
		
	
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $minFrequencyLimit and $maxFrequencyLimit"	\
			-type ok			\
			-icon warning
		return
	}
	

}

#Set Maximum Frequency
#---------------
#This procedure prompts the user for a new max frequency value.
#The frequency supplied by the user is checked to ensure that
#it is a valid number and a valid frequency setting.
proc ::wave::setMaxFrequency {} {
	
	set newMax [Dialog_Prompt setMax "New Maximum Frequency:"]
	
	if {$newMax == ""} { return }
	
	if { [string is double -strict $newMax]} {
		if {$newMax > $wave::minFrequency && $newMax <= $wave::maxFrequencyLimit} {
			set wave::maxFrequency [format "%.1f" $newMax]
			set wavePath [getWavePath]
			::wave::adjustFrequency [$wavePath.freq.freqSlider get]
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Invalid Frequency.\nMax frequency is $wave::maxFrequencyLimit."	\
			-type ok			\
			-icon warning
		}
	} else {
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $wave::minFrequencyLimit and $wave::maxFrequencyLimit."	\
			-type ok			\
			-icon warning
		return
	}
}

#Set Minimum Frequency
#---------------
#This procedure prompts the user for a new min frequency value.
#The frequency supplied by the user is checked to ensure that
#it is a valid number and a valid frequency setting.
proc ::wave::setMinFrequency {} {

	set newMin [Dialog_Prompt setMin "New Minimum Frequency:"]
	
	if {$newMin == ""} {return}
	
	if { [string is double -strict $newMin] } {
		if { $newMin < $wave::maxFrequency && $newMin >= $wave::minFrequencyLimit} {
			set wave::minFrequency [format "%.1f" $newMin]
			set wavePath [getWavePath]
			::wave::adjustFrequency [$wavePath.freq.freqSlider get]
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Invalid Frequency.\nMin frequency is $wave::minFrequencyLimit."	\
			-type ok			\
			-icon warning
		}
	} else {
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $wave::minFrequencyLimit and $wave::maxFrequencyLimit."	\
			-type ok			\
			-icon warning
		return
	}
}

proc ::wave::enableNoise {} {

	set wavePath [getWavePath]
	
	sendCommand "N"

	set plotData {}
	for {set i 0} {$i <256} {incr i} {
		lappend plotData [expr {127+128*(rand()-0.5)}]
	}

	::wave::updateDisplay $plotData

}

proc wave::waveformOutput {} {

	set wavePath [getWavePath]

	if {$wave::outputEnabled} {
		#Make sure all the controls are active
		$wavePath.freq.freqSlider configure -state normal
		$wavePath.amp.ampSlider configure -state normal
		$wavePath.off.offSlider configure -state normal
		#Enable the function generator output
		sendCommand "E"
	} else {
		#Disable the controls
		$wavePath.freq.freqSlider configure -state disabled
		$wavePath.amp.ampSlider configure -state disabled
		$wavePath.off.offSlider configure -state disabled
		#Disable the function generator output
		sendCommand "e"
	}

}

proc wave::loadCustom {} {

	#Get the name of the file
	set fileName [tk_getOpenFile]

	#Make sure the user supplied a file
	if {$fileName ==""} {return}
	
	#Attempt to open the file
	if [catch {open $fileName r} channel] {
		tk_messageBox	\
			-message "Cannot open $fileName"	\
			-default ok	\
			-icon error	\
			-title "File Error"
		puts stderr "Cannot open $fileName"
	} else {
		
		set sampleIndex 0
		set samples [list]
		
		while { [gets $channel line] >= 0} {
		
			#Make sure the sample is a valid number
			if {[string is integer -strict $line] != 1} {
				tk_messageBox	\
					-message "Sample on line $sampleIndex of $fileName is not a valid sample value.\nValid values are 0 to 255."	\
					-default ok	\
					-icon error	\
					-title "Sample Value Error"
				puts stderr "Sample on line $sampleIndex of $fileName is not a valid sample value.\nValid values are 0 to 255."
				close $channel
				return
			}
			
			#Make sure the sample is between 0 and 255
			if { ($line < 0) || ($line > 255)} {
				tk_messageBox	\
					-message "Sample on line $sampleIndex of $fileName is not a valid sample value.\nValid values are 0 to 255."	\
					-default ok	\
					-icon error	\
					-title "Sample Value Error"
				puts stderr "Sample on line $sampleIndex of $fileName is not a valid sample value.\nValid values are 0 to 255."
				close $channel
				return
			}
			
			lappend samples $line
			incr sampleIndex
			
			if {$sampleIndex > 256} {
				tk_messageBox	\
					-message "File is too long.  File: $fileName should contain exactly 256 samples."	\
					-default ok	\
					-icon error	\
					-title "Sample Count Error"
				puts stderr "File is too long.  File: $fileName should contain exactly 256 samples."
				close $channel
				return
			}
		
		}
		close $channel
		
		wave::programCustom $samples
		
	}
		

}

proc wave::programCustom {sampleData} {
	
	set sampleIndex 0
	set wave::customWaveform {}
	while {$sampleIndex < 256} {
		set sampleValue [lindex $sampleData $sampleIndex]
		
		sendCommand "S $sampleIndex $sampleValue"
		lappend wave::customWaveform $sampleValue
		incr sampleIndex
		update
	}
		
	wave::updateDisplay $wave::customWaveform
}

proc wave::saveCustom {} {

	if {[llength $wave::customWaveform] < 256} {
		tk_messageBox	\
			-message "Please load a custom waveform before attempting\nto store the waveform."	\
			-default ok	\
			-icon error	\
			-title "Store Waveform..."
		return
	}
				
	#Load the waveform into the device buffer
	set address 0
	foreach sample $wave::customWaveform {
		sendCommand "J $address $sample"
		incr address
	}
	
	#Save the buffer into non-volatile memory
	sendCommand Q
	
	tk_messageBox	\
		-message "The current custom waveform has been saved\nto the device."	\
		-default ok	\
		-icon info	\
		-title "Store Waveform..."
			
}

# Update Waveform Display
#-------------------------------
#This procedure updates the waveform display by drawing the waveform supplied
#by "plotData" on the canvas.  Data in the plotData array should be x y pairs.
proc ::wave::updateDisplay {sampleData} {

	if {[llength $sampleData]==1} {
		switch $sampleData {
			"0" {
				#Standard Sine Wave
				set sampleData {}
				for {set i 0} {$i <256} {incr i} {
					lappend sampleData [expr {-1*sin(2*3.14159265*(1/255.0)*$i)*128+127}]
				}
			} "1" {
				#Triangle Wave
				set sampleData {}
				for {set i 0} {$i < 64} {incr i} {
					lappend sampleData [expr {128-$i*2}]
				}
				for {set i 64} {$i < 192} {incr i} {
					lappend sampleData [expr {($i-64)*2}]
				}
				for {set i 192} {$i <256} {incr i} {
					lappend sampleData [expr {255-($i-192)*2}]
				}
			} "2" {
				#Square Wave
				set sampleData {}
				for {set i 0} {$i < 128} {incr i} {
					lappend sampleData 0
				}
				for {set i 128} {$i < 256} {incr i} {
					lappend sampleData 255
				}
			} "3" {
				#Sawtooth
				set sampleData {}
				for {set i 0} {$i < 256} {incr i} {
					lappend sampleData [expr {255-$i}]
				}
			} default {
				return
			}
		}
	}

	#Update the waveform display
	set plotData {}
	set scaleFactor [expr {266.0/($wave::displaySize*0.9)}]
	#Get a few samples from the "previous" cycle
	for {set i 250} {$i < 256} {incr i} {
		#X-Coordinate
		lappend plotData [expr {$wave::displaySize*0.05+($i-250)/$scaleFactor}]
		#Y-Coordinate
		set sample [lindex $sampleData $i]
		lappend plotData [expr {$wave::displaySize/2.0+(($sample-128)/$scaleFactor)}]
	}
	#Draw one complete cycle of the waveform
	for {set i 0} {$i <256} {incr i} {
		#X-Coordinate
		lappend plotData [expr {$wave::displaySize*0.05+(5/$scaleFactor)+$i/$scaleFactor}]
		#Y-Coordinate
		set sample [lindex $sampleData $i]
		lappend plotData [expr {$wave::displaySize/2.0+(($sample-128)/$scaleFactor)}]
	}
	#Get a few samples from the "next" cycle
	for {set j 0} {$j < 6} {incr j} {
		#X-Coordinate
		lappend plotData [expr {$wave::displaySize*0.05+$wave::displaySize*0.05/$scaleFactor+$j/$scaleFactor+$i/$scaleFactor}]
		#Y-Coordinate
		set sample [lindex $sampleData $j]
		lappend plotData [expr {$wave::displaySize/2.0+(($sample-128)/$scaleFactor)}]
	}
	
	set wavePath [getWavePath]
	
	$wavePath.wave.waveDisplay delete waveDisplayData
	
	$wavePath.wave.waveDisplay create line	\
		$plotData	\
		-tag waveDisplayData	\
		-fill red
	
}

proc wave::sweepControl {} {
	variable frequencyDisplay

	if {$wave::sweepEnabled} {
		if {$wave::sweepBehavior == "cycle"} {
			if {$wave::sweepDirection == "down"} {
				set wave::sweepInitialFrequency $wave::maxFrequency
			} else {
				set wave::sweepInitialFrequency $wave::minFrequency
			}
		} else {
			if {($wave::sweepBehavior=="upReset")||($wave::sweepBehavior=="upRepeat")} {
				set wave::sweepDirection "up"
				set wave::sweepInitialFrequency $wave::minFrequency
			} else {
				set wave::sweepDirection "down"
				set wave::sweepInitialFrequency $wave::maxFrequency
			}
		}
		sendFrequency $wave::sweepInitialFrequency
		set frequencyDisplay [format "%.1f" $wave::sweepInitialFrequency]
		set frequencyDisplay "$frequencyDisplay Hz"
		if {$wave::sweepTrigOutEnable} {
			[pwm::getPWMPath].ch2.outputControl2.enabled invoke
		}
		wave::sweepService
	} else {
		after cancel $wave::sweepHandle
		if {$wave::sweepTrigOutEnable} {
			[pwm::getPWMPath].ch2.outputControl2.disabled invoke
		}
	}
	
}

proc wave::sweepService {} {
	variable frequencyDisplay
	variable sliderMode
	variable minFrequency
	variable maxFrequency
	variable sliderRange

	if {!$wave::sweepEnabled} {
		after cancel $wave::sweepHandle
		return
	}
	
	#Get the current frequency
	set frequency [lindex $wave::frequencyDisplay 0]
	
	puts "Frequency in is $frequency"
	
	if {$wave::sweepMode == "linear"} {
		if {$wave::sweepDirection == "up"} {
			set frequency [expr {$frequency + $wave::sweepLinearStep}]
			if {$frequency > $maxFrequency} {
				set frequency $maxFrequency
				if {$wave::sweepBehavior=="cycle"} {
					set wave::sweepDirection down
					if {$wave::sweepTrigOutEnable} {
						[pwm::getPWMPath].ch2.outputControl2.disabled invoke
					}
				}
			}
		} else {
			set frequency [expr {$frequency - $wave::sweepLinearStep}]
			if {$frequency < $minFrequency} {
				set frequency $minFrequency
				if {$wave::sweepBehavior=="cycle"} {
					set wave::sweepDirection up
					if {$wave::sweepTrigOutEnable} {
						[pwm::getPWMPath].ch2.outputControl2.enabled invoke
					}
				}
			}
		}
	} else {
		if {$wave::sweepDirection == "up"} {
			set frequency [expr {$frequency * $wave::sweepLogarithmicStep}]
			if {$frequency > $maxFrequency} {
				set frequency $maxFrequency
				if {$wave::sweepBehavior=="cycle"} {
					set wave::sweepDirection down
					if {$wave::sweepTrigOutEnable} {
						[pwm::getPWMPath].ch2.outputControl2.disabled invoke
					}
				}
			}
		} else {
			set frequency [expr {$frequency / $wave::sweepLogarithmicStep}]
			if {$frequency < $minFrequency} {
				set frequency $minFrequency
				if {$wave::sweepBehavior=="cycle"} {
					set wave::sweepDirection up
					if {$wave::sweepTrigOutEnable} {
						[pwm::getPWMPath].ch2.outputControl2.enabled invoke
					}
				}
			}
		}
	}
	
	#Update the slider and the hardware
	if {$sliderMode == "log"} {
		#Logarithmic interpretation of slider position
		set logMin [expr {log10($minFrequency)}]
		set logMax [expr {log10($maxFrequency)}]
		set b $logMin
		set m [expr {($logMax-$logMin)/($sliderRange-1)}]
		set newSliderPosition [expr {(log10($frequency)-$b)/$m + 1}]
	} else {
		#Linear interpretation of slider position
		set b $minFrequency
		set m [expr {($maxFrequency-$minFrequency)/($sliderRange-1)}]
		set newSliderPosition [expr {($frequency-$b)/$m}]
	}	
	[getWavePath].freq.freqSlider set $newSliderPosition
	
	#Update the frequency display
	set frequencyDisplay [format "%.1f" $frequency]
	set frequencyDisplay "$frequencyDisplay Hz"
	
	#Check for end of up/reset and down/reset cycles
	if {($wave::sweepBehavior=="upReset")&&($frequency==$maxFrequency)} {
		set wave::sweepEnabled 0
		[getWavePath].freq.freqSlider set 0
		if {$wave::sweepTrigOutEnable} {
			[pwm::getPWMPath].ch2.outputControl2.disabled invoke
		}
		if {$wave::sweepTriggerMode=="counter"} {
			counter::resetCounter
			set counter::counterSweepTrigger 1
		}
		return
	}
	if {($wave::sweepBehavior=="downReset")&&($frequency==$minFrequency)} {
		set wave::sweepEnabled 0
		[getWavePath].freq.freqSlider set $wave::sliderRange
		if {$wave::sweepTrigOutEnable} {
			[pwm::getPWMPath].ch2.outputControl2.disabled invoke
		}
		if {$wave::sweepTriggerMode=="counter"} {
			counter::resetCounter
			set counter::counterSweepTrigger 1
		}
		return
	}
	if {($wave::sweepBehavior=="upRepeat")&&($frequency==$maxFrequency)} {
		[getWavePath].freq.freqSlider set 0
		if {$wave::sweepTrigOutEnable} {
			[pwm::getPWMPath].ch2.outputControl2.disabled invoke
			after $wave::sweepTime {[pwm::getPWMPath].ch2.outputControl2.enabled invoke}
		}
	}
	if {($wave::sweepBehavior=="downRepeat")&&($frequency==$minFrequency)} {
		[getWavePath].freq.freqSlider set $wave::sliderRange
		if {$wave::sweepTrigOutEnable} {
			[pwm::getPWMPath].ch2.outputControl2.disabled invoke
			after $wave::sweepTime {[pwm::getPWMPath].ch2.outputControl2.enabled invoke}
		}
	}
	
	set wave::sweepHandle [after $wave::sweepTime wave::sweepService]

}

proc wave::sweepSettings {} {

	if {![winfo exists .sweep]} {
	
		toplevel .sweep
		wm title .sweep "Frequency Sweep Settings"
		
		#Create a frame for the sweep mode
		frame .sweep.mode	\
			-relief groove	\
			-borderwidth 2
			
		label .sweep.mode.title	\
			-text "Sweep Mode:"	\
			-font {-weight bold -size -12}
			
		radiobutton .sweep.mode.linear	\
			-text "Linear"	\
			-variable wave::sweepMode	\
			-value linear
			
		radiobutton .sweep.mode.log	\
			-text "Logarithmic"	\
			-variable wave::sweepMode	\
			-value log
			
		grid .sweep.mode.title -row 0 -sticky we
		grid .sweep.mode.linear -row 1 -sticky w
		grid .sweep.mode.log -row 2 -sticky w
		
		frame .sweep.time	\
			-borderwidth 2	\
			-relief groove
		
		label .sweep.time.title	\
			-text "Frequency Time Step \[ms\]"	\
			-font {-weight bold -size -12}
		
		scale .sweep.time.slider	\
			-from 50		\
			-to 5000		\
			-length 100		\
			-showvalue 1	\
			-orient horizontal	\
			-variable wave::sweepTime	\
			-tickinterval 1	\
			-resolution 10
		
		grid .sweep.time.title -row 0
		grid .sweep.time.slider -row 1
			
		frame .sweep.step	\
			-borderwidth 2	\
			-relief groove
		
		label .sweep.step.title	\
			-text "Frequency Step Settings:"	\
			-font {-weight bold -size -12}
			
		label .sweep.step.linearLabel	\
			-text "Linear Step:"
		
		button .sweep.step.linearStep	\
			-textvariable wave::sweepLinearStep	\
			-command {set wave::sweepLinearStep [wave::newStep $wave::sweepLinearStep linear]}	\
			-width 10	
			
		label .sweep.step.logLabel	\
			-text "Logarithmic Step:"
			
		button .sweep.step.logStep	\
			-textvariable wave::sweepLogarithmicStep	\
			-command {set wave::sweepLogarithmicStep [wave::newStep $wave::sweepLogarithmicStep log]}	\
			-width 10
			
		grid .sweep.step.title -row 0 -column 0 -columnspan 2
		grid .sweep.step.linearLabel -row 1 -column 0
		grid .sweep.step.linearStep -row 1 -column 1
		grid .sweep.step.logLabel -row 2 -column 0
		grid .sweep.step.logStep -row 2 -column 1
		
		frame .sweep.setup	\
			-borderwidth 2	\
			-relief groove
			
		label .sweep.setup.title	\
			-text "Initial Sweep Direction:"	\
			-font {-weight bold -size -12}
			
		radiobutton .sweep.setup.up	\
			-text "Up"	\
			-value up	\
			-variable wave::sweepDirection
			
		radiobutton .sweep.setup.down	\
			-text "Down"	\
			-value down	\
			-variable wave::sweepDirection
			
		label .sweep.setup.behaviorTitle	\
			-text "Sweep Behavior"	\
			-font {-weight bold -size -12}
			
		radiobutton .sweep.setup.cycle	\
			-text "Cycle Up/Down"	\
			-value cycle	\
			-variable wave::sweepBehavior	\
			-command {
				.sweep.setup.up configure -state normal
				.sweep.setup.down configure -state normal
			}
	
		radiobutton .sweep.setup.upReset	\
			-text "Sweep Up, Reset"	\
			-value upReset	\
			-variable wave::sweepBehavior	\
			-command {
				.sweep.setup.up configure -state disabled
				.sweep.setup.down configure -state disabled
				set wave::sweepDirection up
			}
			
		radiobutton .sweep.setup.upRepeat	\
			-text "Sweep Up, Repeat"	\
			-value upRepeat	\
			-variable wave::sweepBehavior	\
			-command {
				.sweep.setup.up configure -state disabled
				.sweep.setup.down configure -state disabled
				set wave::sweepDirection up
			}
			
		radiobutton .sweep.setup.downReset	\
			-text "Sweep Down, Reset"	\
			-value downReset	\
			-variable wave::sweepBehavior	\
			-command {
				.sweep.setup.up configure -state disabled
				.sweep.setup.down configure -state disabled
				set wave::sweepDirection down
			}
		
		radiobutton .sweep.setup.downRepeat	\
			-text "Sweep Down, Repeat"	\
			-value downRepeat	\
			-variable wave::sweepBehavior	\
			-command {
				.sweep.setup.up configure -state disabled
				.sweep.setup.down configure -state disabled
				set wave::sweepDirection down
			}

		grid .sweep.setup.title -row 0 -sticky w
		grid .sweep.setup.up -row 1 -sticky w
		grid .sweep.setup.down -row 2 -sticky w
		grid .sweep.setup.behaviorTitle -row 3 -sticky w
		grid .sweep.setup.cycle -row 4 -sticky w
		grid .sweep.setup.upReset -row 5 -sticky w
		grid .sweep.setup.upRepeat -row 6 -sticky w
		grid .sweep.setup.downReset -row 7 -sticky w
		grid .sweep.setup.downRepeat -row 8 -sticky w
		
		frame .sweep.trigger	\
			-borderwidth 2	\
			-relief groove
			
		label .sweep.trigger.title	\
			-text "Sweep Trigger"	\
			-font {-weight bold -size -12}
			
		radiobutton .sweep.trigger.manual	\
			-text "Manual Enable/Disable"	\
			-value manual	\
			-variable wave::sweepTriggerMode
			
		radiobutton .sweep.trigger.counter	\
			-text "Counter Trigger Input"	\
			-value counter	\
			-variable wave::sweepTriggerMode	\
			-command {
				[counter::getCounterPath].modeControls.counterMode invoke
				[counter::getCounterPath].reset invoke
				set counter::counterSweepTrigger 1
			}
		
		checkbutton .sweep.trigger.pwmTriggerOut	\
			-text "Enable Sweep Trigger Out (PWM2)"	\
			-variable wave::sweepTrigOutEnable	\
			-command {
				set pwmPath [getPWMPath]
				if {$wave::sweepTrigOutEnable} {
					$pwmPath.channelSelect invoke buttonup
					$pwmPath.ch2.duty2.dutySlider set 100
					set pwm::minFrequency2 250
					set pwm::maxFrequency2 250.1
					pwm::adjustFrequency 2 [$pwmPath.ch2.freq2.freqSlider get]
					$pwmPath.ch2.outputControl2.disabled invoke
				} else {
					$pwmPath.channelSelect set "Pulse Channel 2"
					pwm::selectPWM
					$pwmPath.ch2.duty2.dutySlider set 0
				}
			}
				
		grid .sweep.trigger.title -row 0 -column 0 -sticky w
		grid .sweep.trigger.manual -row 1 -column 0 -sticky w
		grid .sweep.trigger.counter -row 2 -column 0 -sticky w
		grid .sweep.trigger.pwmTriggerOut -row 3 -column 0 -sticky w
			
		grid .sweep.mode -row 0 -sticky we
		grid .sweep.time -row 1 -sticky we
		grid .sweep.step -row 2 -sticky we
		grid .sweep.setup -row 3 -sticky we
		grid .sweep.trigger -row 4 -sticky we
		
		raise .sweep
		focus .sweep
	} else {
		wm deiconify .sweep
		raise .sweep
		focus .sweep
	}

}

proc wave::newStep {oldValue type} {

	set newValue [Dialog_Prompt setStep "New Step Value:"]
	
	if {$newValue == ""} {return $oldValue}
	
	if {![string is double -strict $newValue]} {return $oldValue}
	
	if {$newValue > $wave::maxFrequency} {return $oldValue}
	
	if {($type == "linear") && ($newValue < $wave::minFrequencyLimit)} {return $oldValue}
	
	if {($type == "log") && !($newValue > 1.0)} {return $oldValue}
	
	return $newValue


}
