#File: main.tcl
#Syscomp CircuitGear GUI
#Main File

#JG
set softwareVersion "1.8"
#Copyright 2008 Syscomp Electronic Design
#www.syscompdesign.com

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License as
#published by the Free Software Foundation; either version 2 of
#the License, or (at your option) any later verison.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranty of
#MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
#the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
#USA

#---=== Global Variables ===---
set waveControlEnabled 1
set pwmControlEnabled 1
set patternControlEnabled 1
set counterControlEnabled 1
set triggerControlEnabled 1

#Folder Location for GUI Images
set images "./Images"

#Debug level for printing messages to the console
set debugLevel 0

#Figure out which operating system we're running on
set osType $tcl_platform(platform)
if {$osType == "unix"} {
	if {[exec uname] == "Darwin"} {set osType "Darwin"}
}

#Img package required for screen captures
package require Img

#BWidget package used for comboboxes
package require BWidget

#Status Indicator
for {set i 0} {$i < 8} {incr i} {
	set statusImage($i) [image create photo -file "./Images/Status$i.gif"]
}
set statusState 0

#---=== Includes ===---
source usbSerial.tcl
source functionGen.tcl
source dialog.tcl
source pwm.tcl
source pattern.tcl
source counter.tcl
source waveEditor.tcl
source trigger.tcl
source firmware.tcl

#Bring public commands into the global namespace
namespace import ::usbSerial::*
namespace import ::wave::*
namespace import ::pwm::*
namespace import ::pattern::*
namespace import ::counter::*
namespace import ::waveEdit::*
namespace import ::trigger::*

#---=== Procedures ===---

proc showAbout {} {
	tk_messageBox	\
		-message "Syscomp Electronic Design Ltd.\nWGM-201 GUI Version $::softwareVersion\n$usbSerial::firmwareIdent\nwww.syscompdesign.com\nWavemaker software by John Foster (http://www.jsjf.demon.co.uk/)"	\
		-default ok	\
		-icon info	\
		-title "About"
}

proc showManual {} {

	set scriptPath [file dirname [info script]]
	
	if {$::osType=="windows"} {
		if {[file exists "$scriptPath/../Documentation/circuit-gear-manual.pdf"]} {
			puts "Launching manual from Documenation directory"
			eval exec [auto_execok start] \"\" [list "$scriptPath/../Documentation/wgm-201-manual.pdf"]
		} else {
			puts "Launcing manual from Source directory"
			eval exec [auto_execok start] \"\" [list "wgm-201-manual.pdf"]
		}
	} else {
		eval exec see [list "wgm-201-manual.pdf"]
	}


}

proc showChangeLog {} {

	#Make sure the change log isn't already open
	if {[winfo exists .changeLog]} {
		raise .changeLog
		focus .changeLog
		return
	}
	
	toplevel .changeLog
	wm title .changeLog "WGM-201 Change Log"
	
	set fileId [open "Changes.txt" r]
	set changeData [read $fileId]
	close $fileId
	
	text .changeLog.log	\
		-width 80	\
		-yscrollcommand ".changeLog.scrollVert set"	\
		-xscrollcommand ".changeLog.scrollHor set"	\
		-wrap none
		
	.changeLog.log insert end $changeData
	.changeLog.log configure -state disabled
	
	scrollbar .changeLog.scrollVert	\
		-command ".changeLog.log yview"	\
		-orient vertical
		
	scrollbar .changeLog.scrollHor	\
		-command ".changeLog.log xview"	\
		-orient horizontal
		
	grid .changeLog.log -row 0 -column 0 -sticky news
	grid .changeLog.scrollVert -row 0 -column 1 -sticky ns
	grid .changeLog.scrollHor -row 1 -column 0 -sticky we
	grid rowconfigure .changeLog .changeLog.log -weight 1
	grid rowconfigure .changeLog .changeLog.scrollVert -weight 1
	grid columnconfigure .changeLog .changeLog.log -weight 1
	grid columnconfigure .changeLog .changeLog.scrollHor -weight 1
	
}
	

#---=== End of Procedures ===---

#Build the control panels
setWavePath ".fg"
buildFunctionGen
setPWMPath ".pwm"
buildPWM
setPatternPath ".pattern"
buildPattern
setCounterPath ".counter"
buildCounter
setTriggerPath ".trigger"
buildTrigger


#Standard windows options
wm title . "Syscomp WGM-201"
wm resizable . 0 0

#Create the menu bar
frame .menubar -relief raised -borderwidth 1

#Create the drop down menus

#File Menu
menubutton .menubar.file	\
	-text "File"		\
	-menu .menubar.file.filemenu
menu .menubar.file.filemenu -tearoff 0
.menubar.file.filemenu add command	\
	-label "Exit"	\
	-command {destroy .}

#View Menu
menubutton .menubar.view \
	-text "View"	\
	-menu .menubar.view.viewMenu
menu .menubar.view.viewMenu -tearoff 0
.menubar.view.viewMenu add checkbutton	\
	-label "PWM Controls"	\
	-variable pwmControlEnabled	\
	-command remapGUI
.menubar.view.viewMenu add checkbutton	\
	-label "Pattern Controls"	\
	-variable patternControlEnabled	\
	-command remapGUI
.menubar.view.viewMenu add checkbutton	\
	-label "Counter Controls"	\
	-variable counterControlEnabled	\
	-command remapGUI
.menubar.view.viewMenu add checkbutton	\
	-label "Trigger Controls"	\
	-variable triggerControlEnabled	\
	-command remapGUI
.menubar.view.viewMenu add separator
if {$osType == "windows"} {
	.menubar.view.viewMenu add command	\
		-label "Debug Console"	\
		-command {console show}
}


#Tools Menu
menubutton .menubar.tools	\
	-text "Tools"	\
	-menu .menubar.tools.toolsMenu
menu .menubar.tools.toolsMenu -tearoff 0
#Add WaveMaker command
.menubar.tools.toolsMenu add command	\
	-label "WaveMaker Waveform Editor"	\
	-command waveMaker::showWaveMaker
#Screen capture command
.menubar.tools.toolsMenu add command	\
	-label "Screen Capture (jpg)"		\
	-command capture::showCapture
.menubar.tools.toolsMenu add separator
#Waveform Editor Command
.menubar.tools.toolsMenu add command	\
	-label "Pattern Editor"	\
	-command waveEdit::showWaveEdit
.menubar.tools.toolsMenu add separator
#Sweep Controls
.menubar.tools.toolsMenu add command	\
	-label "Frequency Sweep Settings"	\
	-command wave::sweepSettings
.menubar.tools.toolsMenu add separator
#Firmware Upgrade Command
.menubar.tools.toolsMenu add command	\
	-label "Firmware Upgrade..."	\
	-command firmware::showFirmware
	

#Hardware Menu
menubutton .menubar.hardware	\
	-text "Hardware"	\
	-menu .menubar.hardware.hardwareMenu
menu .menubar.hardware.hardwareMenu	-tearoff 0
.menubar.hardware.hardwareMenu add command	\
	-label "Connect..."	\
	-command ::usbSerial::serialSettings
.menubar.hardware.hardwareMenu add separator

#Help Menu
menubutton .menubar.help	\
	-text "Help"		\
	-menu .menubar.help.helpMenu
menu .menubar.help.helpMenu -tearoff 0
.menubar.help.helpMenu add command	\
	-label "About"	\
	-command showAbout
.menubar.help.helpMenu add separator
.menubar.help.helpMenu add command	\
	-label "Manual (pdf)"	\
	-command showManual
.menubar.help.helpMenu add separator
.menubar.help.helpMenu add command	\
	-label "Change Log"	\
	-command showChangeLog
.menubar.help.helpMenu add separator

#Create an indicator for the status of the serial-usb connection
label .menubar.serialPortStatus	\
	-textvariable ::usbSerial::serialStatus	\
	-background red
label .menubar.status	\
	-image $statusImage(0)
	
#Build buttons for the dynamic GUI
button .menubar.wave	\
	-text "Waveform Generator"	\
	-command {set waveControlEnabled 1;remapGUI}
button .menubar.pwm	\
	-text "PWM"	\
	-command {set pwmControlEnabled 1;remapGUI}
button .menubar.pattern	\
	-text "Pattern"	\
	-command {set patternControlEnabled 1;remapGUI}
button .menubar.counter	\
	-text "Counter"	\
	-command {set counterControlEnabled 1;remapGUI}
button .menubar.trigger	\
	-text "Trigger"	\
	-command {set triggerControlEnabled 1;remapGUI}

#Place the menus on the menubar
grid .menubar.file -row 0 -column 0 -sticky w
grid .menubar.view -row 0 -column 1 -sticky w
grid .menubar.tools -row 0 -column 2 -sticky w
grid .menubar.hardware -row 0 -column 3 -sticky w
grid .menubar.help -row 0 -column 4 -sticky w
grid .menubar.serialPortStatus -row 0 -column 5 -sticky w -padx 15
grid .menubar.status -row 0 -column 6 -sticky w -padx 10


#Place the major Frames
grid .menubar -row 0 -column 0 -sticky w -columnspan 10

proc remapGUI {} {
		
	if {$::waveControlEnabled} {
		grid remove .menubar.wave
		grid .fg -row 2 -column 0
	} else {
		grid remove .fg
		grid .menubar.wave -row 0 -column 7
	}
	if {$::pwmControlEnabled} {
		grid remove .menubar.pwm
		grid .pwm -row 2 -column 1
	} else {
		grid remove .pwm
		grid .menubar.pwm -row 0 -column 8
	}
	if {$::patternControlEnabled} {
		grid remove .menubar.pattern
		grid .pattern -row 2 -column 2
	} else {
		grid remove .pattern
		grid .menubar.pattern -row 0 -column 9
	}
	if {$::counterControlEnabled} {
		grid remove .menubar.counter
		grid .counter -row 2 -column 3
	} else {
		grid remove .counter
		grid .menubar.counter -row 0 -column 10
	}
	if {$::triggerControlEnabled} {
		grid remove .menubar.trigger
		grid .trigger -row 2 -column 4 -sticky n
	} else {
		grid remove .trigger
		grid .menubar.trigger -row 0 -column 11
	}
}

remapGUI


if [catch {open port.cfg r+} fileId] {
	if {$osType == "windows"} {
		set usbSerial::serialPort "com4:"
	} else {
		set serialPort "/dev/ttyUSB0"
	}
} else {
	if { [gets $fileId line] >= 0} {
		set usbSerial::serialPort $line
	}
	close $fileId
}

proc initializeWGM {} {

	set wavePath [getWavePath]
	set pwmPath [getPWMPath]
	set patternPath [getPatternPath]
	set counterPath [getCounterPath]

	#Change the generator frequency to ~250Hz
	$wavePath.freq.freqSlider set 160
	
	#Make sure the waveform output is enabled
	sendCommand E
	#Select a sine wave as the default output
	sendCommand "n"
	sendCommand "W 0"
	
	#Make sure the PWM Outputs are enabled
	sendCommand 1
	sendCommand 2
	
	#Default PWM Frequency
	$pwmPath.ch1.freq1.freqSlider set [expr {$pwm::sliderRange/2}]
	$pwmPath.ch2.freq2.freqSlider set [expr {$pwm::sliderRange/2}]
	$pwmPath.ch1.duty1.dutySlider set 50
	$pwmPath.ch2.duty2.dutySlider set 50
	
	#Make sure the pattern generator is enabled and set the outputs to zero
	sendCommand "B 0 0 0"
	sendCommand "B 0 1 0"
	sendCommand "l 9"
	sendCommand 3
	
	#Default Pattern Generator Frequency
	$patternPath.freq.freqSlider set [expr {$pattern::sliderRange/2}]
	
	
	
	#Default counter trigger level
	$counterPath.trigger.triggerLevel set 2.5
	
	#Initialize the trigger controls
	trigger::selectWaveTriggerMode
	trigger::selectWaveOutputMode
	trigger::selectPatternTriggerMode
	trigger::updateCounterThreshold
	trigger::updateOnCycles
	trigger::updateOffCycles
	trigger::updateRepeatCycles
	trigger::selectTriggerSource
	
	#Initialize Counter
	counter::selectMode
	counter::counterRequest
	
}



openSerialPort

#Add-Ons
source capture.tcl
source updateCheck.tcl
source wavemaker.tcl



