
package provide pattern 1.0
package require Img

namespace eval pattern {

#---=== Pattern Generator Global Variables ===---
set minFrequency 0.1
set maxFrequency 20000
set sliderRange 327
set ddsResolution 0.031247735
set minFrequencyLimit $ddsResolution
set absFrequencyLimit 20000
set maxFrequencyLimit 20000
set defaultFrequency 1000.0
set frequencyDisplay $defaultFrequency
set outputEnabled 1
set patternLength 2


#---=== Export Public Procedures ===---
namespace export buildPattern
namespace export setPatternPath
namespace export getPatternPath
}

#---=== Procedures ===---
proc pattern::setPatternPath {patternPath} {
	variable pattern
	
	set pattern(path) $patternPath
}

proc pattern::getPatternPath {} {
	variable pattern
	
	return $pattern(path)
}


proc pattern::buildPattern {} {


	set patternPath [pattern::getPatternPath]
	
	#Frame for Pattern Controls
	frame $patternPath	\
		-relief raised	\
		-borderwidth 2
		
	label $patternPath.title	\
		-text "Pattern Generator"	\
		-font {-weight bold -size -16}
	
	button $patternPath.hide	\
		-text "X"	\
		-font {-weight bold -size -12}	\
		-relief flat	\
		-command {set patternControlEnabled 0;remapGUI}
	
	#Pattern frequency controls
	frame $patternPath.freq	\
		-relief groove	\
		-borderwidth 2
		
	scale $patternPath.freq.freqSlider	\
		-from $pattern::sliderRange	\
		-to 1		\
		-variable pattern::frequencyPosition	\
		-orient vertical	\
		-tickinterval 0	\
		-resolution 1	\
		-showvalue 0	\
		-length $pattern::sliderRange	\
		-command "pattern::adjustFrequency"
		
	label $patternPath.freq.title	\
		-text "Frequency"	\
		-font {-weight bold -size -12}

	button $patternPath.freq.topValue	\
		-textvariable pattern::maxFrequency	\
		-width 11	\
		-command "pattern::setMaxFrequency"
		
	button $patternPath.freq.bottomValue	\
		-textvariable pattern::minFrequency	\
		-width 11	\
		-command "pattern::setMinFrequency"

	grid $patternPath.freq.title -row 0 -column 0 -pady 6
	grid $patternPath.freq.topValue -row 1 -column 0
	grid $patternPath.freq.freqSlider -row 2 -column 0
	grid $patternPath.freq.bottomValue -row 3 -column 0
	
	#Pattern Generator Output Controls
	frame $patternPath.outputControl	\
		-relief groove	\
		-borderwidth 2
		
	label $patternPath.outputControl.title	\
		-text "Pattern Output:"
	
	radiobutton $patternPath.outputControl.enabled	\
		-text "Enabled"	\
		-value 1	\
		-variable pattern::outputEnabled	\
		-command pattern::patternOutput

	$patternPath.outputControl.enabled select
		
	radiobutton $patternPath.outputControl.disabled	\
		-text "Disabled"	\
		-value 0	\
		-variable pattern::outputEnabled	\
		-command pattern::patternOutput
		
	grid $patternPath.outputControl.title -row 0 -column 0 -sticky nw
	grid $patternPath.outputControl.enabled -row 1 -column 0 -sticky nw
	grid $patternPath.outputControl.disabled -row 2 -column 0 -sticky nw
	
	#Sweep Controls
	frame $patternPath.sweep	\
		-relief groove	\
		-borderwidth 2
		
	label $patternPath.sweep.modeTitle	\
		-text "Sweep Mode:"
		
	radiobutton $patternPath.sweep.logMode	\
		-text "Logarithmic"	\
		-value log	\
		-variable pattern::sliderMode
		
	#Select Logarithmic Mode by Default
	$patternPath.sweep.logMode select
	
	radiobutton $patternPath.sweep.linMode	\
		-text "Linear"	\
		-value linear	\
		-variable pattern::sliderMode
		
	grid $patternPath.sweep.modeTitle -row 0 -column 0 -sticky nw
	grid $patternPath.sweep.logMode -row 1 -column 0 -sticky nw
	grid $patternPath.sweep.linMode -row 2 -column 0 -sticky nw

	#Frequency Display
	button $patternPath.freqDisplay	\
		-relief sunken	\
		-borderwidth 3	\
		-textvariable pattern::frequencyDisplay	\
		-font {-weight bold -size -14}	\
		-width 10	\
		-background black	\
		-foreground red	\
		-command pattern::setFrequency
		
	#Sample Time Display
	label $patternPath.sampleDisplay	\
		-relief sunken	\
		-borderwidth 3	\
		-textvariable pattern::sampleDisplay	\
		-font {-weight bold -size -14}	\
		-width 10	\
		-background black	\
		-foreground green
		
	button $patternPath.loadWaveform	\
		-text "Load Pattern"	\
		-command pattern::loadWaveform
		
	button $patternPath.editWaveform	\
		-text "Edit Pattern"	\
		-command waveEdit::showWaveEdit
		
	grid $patternPath.title -row 0 -column 0 -columnspan 2 -sticky w
	grid $patternPath.hide -row 0 -column 1 -sticky ne
	grid $patternPath.freqDisplay -row 1 -column 0 -sticky we -columnspan 2
	grid $patternPath.sampleDisplay -row 2 -column 0 -sticky we -columnspan 2
	grid $patternPath.loadWaveform -row 3 -column 0
	grid $patternPath.editWaveform -row 3 -column 1
	grid $patternPath.freq -row 4 -column 0 -columnspan 2
	grid $patternPath.sweep -row 5 -column 0
	grid $patternPath.outputControl -row 5 -column 1
	

}

proc pattern::adjustFrequency {sliderArg} {
	variable minFrequency
	variable maxFrequency
	variable sliderRange
	variable frequencyDisplay
	variable sampleDisplay
	variable sliderMode
	
	
	if {$sliderMode == "log"} {
		#Logarithmic interpretation of slider position
		set logMin [expr {log10($minFrequency)}]
		set logMax [expr {log10($maxFrequency)}]
		set b $logMin
		set m [expr {($logMax-$logMin)/($sliderRange-1)}]
		set y [expr {$m*($sliderArg-1)+$b}]
		set frequency [expr {pow(10,$y)}]
		
	} else {
		#Linear interpretation of slider position
		set b $minFrequency
		set m [expr {($maxFrequency-$minFrequency)/($sliderRange-1)}]
		set y [expr {$m*($sliderArg-1)+$b}]
		set frequency $y
	}
	
	#Round to the nearest thousands of a hertz
	set waveFrequency [format "%.3f" $frequency]
	
	#Update the hardware with the new frequency
	pattern::sendFrequency $waveFrequency
	
	#Update the frequency display
	set frequencyDisplay "$waveFrequency Hz"
	set sampleDisplay [pattern::formatTime [expr {1.0/$waveFrequency/$pattern::patternLength}]]
	
}

# Send Frequency Setting to Hardware
#-------------------------------------------
# This procedure accepts a frequency value, converts it to a four byte number,
# and sends it to the hardware.
proc pattern::sendFrequency {freq} {
	variable ddsResolution
	
	#Calculate the phase integer
	set freqOutput [expr {round($freq/($ddsResolution*1024/$pattern::patternLength))}]

	#Break the phase integer into a four byte number
	set byte3 [expr {round(floor($freqOutput/pow(2,24)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,24))}]
	
	set byte2 [expr {round(floor($freqOutput/pow(2,16)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,16))}]
	
	set byte1 [expr {round(floor($freqOutput/pow(2,8)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,8))}]
	
	set byte0 $freqOutput
	
	sendCommand "H $byte3 $byte2 $byte1 $byte0"
	
}

# Manually Set Frequency
#---------------------------
# This procedure is called when the user wants to manually set the waveform generator
# output frequency.  It presents the user with a dialog box where they can enter
# the desired output frequency.
proc pattern::setFrequency {} {
	variable minFrequencyLimit
	variable maxFrequencyLimit
	variable frequencyDisplay
	variable sampleDisplay
	
	#Dialog box for user to enter the new frequency
	set newFreq [Dialog_Prompt newF "New Frequency:"]
	
	if {$newFreq == ""} {return}
	
	#Make sure that we got a valid frequency setting
	if { [string is double -strict $newFreq] } {
		if { $newFreq >= $minFrequencyLimit && $newFreq <= $maxFrequencyLimit} {
			sendFrequency $newFreq
			set frequencyDisplay [format "%.3f" $newFreq]
			set sampleDisplay [pattern::formatTime [expr {1.0/$newFreq/$pattern::patternLength}]]
			set frequencyDisplay "$frequencyDisplay Hz"
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency out of range"	\
			-type ok			\
			-icon warning
		}
	} else {
		
	
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $minFrequencyLimit and $maxFrequencyLimit"	\
			-type ok			\
			-icon warning
		return
	}
	

}

#Set Maximum Frequency
#---------------
#This procedure prompts the user for a new max frequency value.
#The frequency supplied by the user is checked to ensure that
#it is a valid number and a valid frequency setting.
proc pattern::setMaxFrequency {} {
	
	set newMax [Dialog_Prompt setMax "New Maximum Frequency:"]
	
	if {$newMax == ""} { return }
	
	if { [string is double -strict $newMax]} {
		if {$newMax > $pattern::minFrequency && $newMax <= $pattern::maxFrequencyLimit} {
			set pattern::maxFrequency [format "%.3f" $newMax]
			set patternPath [getPatternPath]
			pattern::adjustFrequency [$patternPath.freq.freqSlider get]
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Invalid Frequency.\nMax frequency is $pattern::maxFrequencyLimit."	\
			-type ok			\
			-icon warning
		}
	} else {
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $pattern::minFrequencyLimit and $pattern::maxFrequencyLimit."	\
			-type ok			\
			-icon warning
		return
	}
}

#Set Minimum Frequency
#---------------
#This procedure prompts the user for a new min frequency value.
#The frequency supplied by the user is checked to ensure that
#it is a valid number and a valid frequency setting.
proc pattern::setMinFrequency {} {

	set newMin [Dialog_Prompt setMin "New Minimum Frequency:"]
	
	if {$newMin == ""} {return}
	
	if { [string is double -strict $newMin] } {
		if { $newMin < $pattern::maxFrequency && $newMin >= $pattern::minFrequencyLimit} {
			set pattern::minFrequency [format "%.3f" $newMin]
			set patternPath [getPatternPath]
			pattern::adjustFrequency [$patternPath.freq.freqSlider get]
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Invalid Frequency.\nMin frequency is $pattern::minFrequencyLimit."	\
			-type ok			\
			-icon warning
		}
	} else {
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $pattern::minFrequencyLimit and $pattern::maxFrequencyLimit."	\
			-type ok			\
			-icon warning
		return
	}
}


proc pattern::patternOutput {} {

	set patternPath [getPatternPath]

	if {$pattern::outputEnabled} {
		#Make sure all the controls are active
		$patternPath.freq.freqSlider configure -state normal
		#Enable the function generator output
		sendCommand "3"
	} else {
		#Disable the controls
		$patternPath.freq.freqSlider configure -state disabled
		#Disable the function generator output
		sendCommand "#"
	}

}

proc pattern::loadWaveform {} {

	#Get the name of the file
	set fileName [tk_getOpenFile]

	#Make sure the user supplied a file
	if {$fileName ==""} {return}
	
	#Attempt to open the file
	if [catch {open $fileName r} channel] {
		tk_messageBox	\
			-message "Cannot open $fileName"	\
			-default ok	\
			-icon error	\
			-title "File Error"
		puts stderr "Cannot open $fileName"
	} else {
		
		set sampleIndex 0
		set samples [list]
		
		#Read the pattern length from the file
		if {[gets $channel line] >= 0} {
			#Make sure the record length is a number
			if {[string is integer -strict $line]!=1} {
				tk_messageBox	\
					-message "Unable to get record length from file.\n"	\
					-default ok	\
					-icon error	\
					-title "File Error"
				close $channel
				return
			}
			#Make sure the record length is a power of 2
			if {[expr {$line%2}]} {
				tk_messageBox	\
					-message "Invalid record length in file.\nRecord length must be a power of 2."	\
					-default ok	\
					-icon error	\
					-title "Record Error"
				close $channel
				return
			}
			#Make sure the record length is in the valid range
			if {($line < 2)||($line > 1024)} {
				tk_messageBox	\
					-message "Invalid record length in file.\nRecord length must be between 2 and 1024."	\
					-default ok	\
					-icon error	\
					-title "Record Error"
				close $channel
				return
			}
		} else {
			tk_messageBox \
				-message "Unable to read file.\n"	\
				-default ok	\
				-icon error	\
				-title "File Error"
			close $channel
			return
		}
		
		set recordLength $line
			
			
		while { [gets $channel line] >= 0} {
		
			#Make sure the sample is a valid number
			if {[string is integer -strict $line] != 1} {
				tk_messageBox	\
					-message "Sample on line $sampleIndex of $fileName is not a valid sample value.\nValid values are 0 to 255."	\
					-default ok	\
					-icon error	\
					-title "Sample Value Error"
				puts stderr "Sample on line $sampleIndex of $fileName is not a valid sample value.\nValid values are 0 to 255."
				close $channel
				return
			}
			
			#Make sure the sample is between 0 and 255
			if { ($line < 0) || ($line > 255)} {
				tk_messageBox	\
					-message "Sample on line $sampleIndex of $fileName is not a valid sample value.\nValid values are 0 to 255."	\
					-default ok	\
					-icon error	\
					-title "Sample Value Error"
				puts stderr "Sample on line $sampleIndex of $fileName is not a valid sample value.\nValid values are 0 to 255."
				close $channel
				return
			}
			
			lappend samples $line
			incr sampleIndex
			
			if {$sampleIndex > $recordLength} {
				tk_messageBox	\
					-message "File is too long.  File: $fileName should contain exactly $recordLength samples."	\
					-default ok	\
					-icon error	\
					-title "Sample Count Error"
				puts stderr "File is too long.  File: $fileName should contain exactly $recordLength samples."
				close $channel
				return
			}
		
		}
		close $channel
		
		#Make sure there are enough samples in the file
		if {$sampleIndex < $recordLength} {
			tk_messageBox	\
				-message "File is too short.  File: $fileName should contain exactly $recordLength samples."	\
				-default ok	\
				-icon error	\
				-title "Sample Count Error"
			puts stderr "File is too short.  File: $fileName should contain exactly $recordLength samples."

			return
		}
		
		#Program the waveform in the device
		set sampleIndex 0
		while {$sampleIndex < $recordLength} {
			set sampleValue [lindex $samples $sampleIndex]
			
			set byte1 [expr {round(floor($sampleIndex/pow(2,8)))}]
			set byte0 [expr {$sampleIndex%round(pow(2,8))}]
			
			sendCommand "B $byte1 $byte0 $sampleValue"
			incr sampleIndex

			update
		}
		
		#Set pattern length
		switch $recordLength {
			"1024" {sendCommand "l 0"}
			"512" {sendCommand "l 1"}
			"256" {sendCommand "l 2"}
			"128" {sendCommand "l 3"}
			"64" {sendCommand "l 4"}
			"32" {sendCommand "l 5"}
			"16" {sendCommand "l 6"}
			"8" {sendCommand "l 7"}
			"4" {sendCommand "l 8"}
			"2" {sendCommand "l 9"}
		}
		set pattern::patternLength $recordLength
		
		set pattern::maxFrequencyLimit [expr {$pattern::absFrequencyLimit*1024/$recordLength}]
		set pattern::maxFrequency [format "%.3f" $pattern::maxFrequencyLimit]
		set patternPath [getPatternPath]
		pattern::adjustFrequency [$patternPath.freq.freqSlider get]
		
		puts "Pattern waveform loaded."
	}
		

}

proc pattern::formatTime {period} {

	if {$period < 1E-6} {
		set period [format "%3.2f" [expr $period/1E-9]]
		set period "$period ns/sample"
	} elseif {$period < 1E-3} {
		set period [format "%3.2f" [expr $period/1E-6]]
		set period "$period us/sample"
	} elseif {$period < 1.0} {
		set period [format "%3.2f" [expr $period/1E-3]]
		set period "$period ms/sample"
	} else {
		set period [format "%3.2f" $period]
		set period "$period s/sample"
	}
	return $period


}

set pattern::sampleDisplay [pattern::formatTime [expr {1.0/$pattern::defaultFrequency/$pattern::patternLength}]]