
package provide pwm 1.0
package require Img

namespace eval pwm {

#---=== PWM Global Variables ===---
set minFrequencyLimit 0.1
set maxFrequencyLimit 100000
set minFrequency1 0.1
set minFrequency2 0.1
set maxFrequency1 100000
set maxFrequency2 100000
set sliderRange 350
set ddsResolution 0.031247735
set defaultFrequency 1000.0
set frequencyDisplay1 $defaultFrequency
set frequencyDisplay2 $defaultFrequency
set outputEnabled1 1
set outputEnabled2 1
set pwmSliderLength 410
set duty1 0
set duty2 0
set pwmChannel "Pulse Channel 1"

#---=== Export Public Procedures ===---
namespace export buildPWM
namespace export setPWMPath
namespace export getPWMPath
}

#---=== Procedures ===---
proc pwm::setPWMPath {pwmPath} {
	variable pwm
	
	set pwm(path) $pwmPath
}

proc pwm::getPWMPath {} {
	variable pwm
	
	return $pwm(path)
}


proc pwm::buildPWM {} {


	set pwmPath [pwm::getPWMPath]
	
	#Frame for PWM Controls
	frame $pwmPath	\
		-relief raised	\
		-borderwidth 2
		
	label $pwmPath.title	\
		-text "Pulse Controls"	\
		-font {-weight bold -size -16}
	
	button $pwmPath.hide	\
		-text "X"	\
		-font {-weight bold -size -12}	\
		-relief flat	\
		-command {set pwmControlEnabled 0;remapGUI}
		
	spinbox $pwmPath.channelSelect	\
		-values {"Pulse Channel 1" "Pulse Channel 2"}	\
		-textvariable pwm::pwmChannel	\
		-command pwm::selectPWM

	for {set i 1} {$i < 3} {incr i} {
	
		#Frame for this channel's controls
		frame $pwmPath.ch$i	\
			-relief groove	\
			-borderwidth 2
	
		#PWM frequency controls
		frame $pwmPath.ch$i.freq$i	\
			-relief groove	\
			-borderwidth 2
		
		scale $pwmPath.ch$i.freq$i.freqSlider	\
			-from $pwm::sliderRange	\
			-to 1		\
			-variable pwm::frequencyPosition$i	\
			-orient vertical	\
			-tickinterval 0	\
			-resolution 1	\
			-showvalue 0	\
			-length $pwm::sliderRange	\
			-command "pwm::adjustFrequency $i"
		
		label $pwmPath.ch$i.freq$i.title	\
			-text "Frequency"	\
			-font {-weight bold -size -12}

		button $pwmPath.ch$i.freq$i.topValue	\
			-textvariable pwm::maxFrequency$i	\
			-width 8	\
			-command "pwm::setMaxFrequency $i"
		
		button $pwmPath.ch$i.freq$i.bottomValue	\
			-textvariable pwm::minFrequency$i	\
			-width 8	\
			-command "pwm::setMinFrequency $i"

		grid $pwmPath.ch$i.freq$i.title -row 0 -column 0 -pady 6
		grid $pwmPath.ch$i.freq$i.topValue -row 1 -column 0
		grid $pwmPath.ch$i.freq$i.freqSlider -row 2 -column 0
		grid $pwmPath.ch$i.freq$i.bottomValue -row 3 -column 0
	
		#PWM Duty Cycle Controls
		frame $pwmPath.ch$i.duty$i	\
			-relief groove	\
			-borderwidth 2
		
		scale $pwmPath.ch$i.duty$i.dutySlider	\
			-from 100		\
			-to 0			\
			-variable pwm::duty$i	\
			-orient vertical	\
			-showvalue 1	\
			-length $pwm::pwmSliderLength	\
			-tickinterval 5	\
			-resolution 1	\
			-command "pwm::adjustDuty $i"
		
		label $pwmPath.ch$i.duty$i.title	\
			-text "Duty Cycle"	\
			-font {-weight bold -size -12}

		grid $pwmPath.ch$i.duty$i.title -row 0 -column 0 -sticky n
		grid $pwmPath.ch$i.duty$i.dutySlider -row 1 -column 0 -sticky n
	
		#PWM Output Controls
		frame $pwmPath.ch$i.outputControl$i	\
			-relief groove	\
			-borderwidth 2
		
		label $pwmPath.ch$i.outputControl$i.title	\
			-text "Pulse $i Output:"
	
		radiobutton $pwmPath.ch$i.outputControl$i.enabled	\
			-text "Enabled"	\
			-value 1	\
			-variable pwm::outputEnabled$i	\
			-command "pwm::pwmOutput $i"
	
		$pwmPath.ch$i.outputControl$i.enabled select
		
		radiobutton $pwmPath.ch$i.outputControl$i.disabled	\
			-text "Disabled"	\
			-value 0	\
			-variable pwm::outputEnabled$i	\
			-command "pwm::pwmOutput $i"
		
		grid $pwmPath.ch$i.outputControl$i.title -row 0 -column 0 -sticky nw
		grid $pwmPath.ch$i.outputControl$i.enabled -row 1 -column 0 -sticky nw
		grid $pwmPath.ch$i.outputControl$i.disabled -row 2 -column 0 -sticky nw
	
		#Sweep Controls
		frame $pwmPath.ch$i.sweep$i	\
			-relief groove	\
			-borderwidth 2
		
		label $pwmPath.ch$i.sweep$i.modeTitle	\
			-text "Sweep Mode:"
		
		radiobutton $pwmPath.ch$i.sweep$i.logMode	\
			-text "Logarithmic"	\
			-value log	\
			-variable pwm::sliderMode$i
		
		#Select Logarithmic Mode by Default
		$pwmPath.ch$i.sweep$i.logMode select
	
		radiobutton $pwmPath.ch$i.sweep$i.linMode	\
			-text "Linear"	\
			-value linear	\
			-variable pwm::sliderMode$i
		
		grid $pwmPath.ch$i.sweep$i.modeTitle -row 0 -column 0 -sticky nw
		grid $pwmPath.ch$i.sweep$i.logMode -row 1 -column 0 -sticky nw
		grid $pwmPath.ch$i.sweep$i.linMode -row 2 -column 0 -sticky nw
	
		#Frequency Display
		button $pwmPath.ch$i.freqDisplay$i	\
			-relief sunken	\
			-borderwidth 3	\
			-textvariable pwm::frequencyDisplay$i	\
			-font {-weight bold -size -14}	\
			-width 10	\
			-background black	\
			-foreground red	\
			-command "pwm::setFrequency $i"
		
		grid $pwmPath.ch$i.freqDisplay$i -row 0 -column 0 -columnspan 3 -sticky we
		grid $pwmPath.ch$i.freq$i -row 1 -column 0
		grid $pwmPath.ch$i.duty$i -row 1 -column 1
		grid $pwmPath.ch$i.sweep$i -row 2 -column 0
		grid $pwmPath.ch$i.outputControl$i -row 2 -column 1
	}
	
	#PWM Controls
	grid $pwmPath.title -row 0 -column 0 -sticky w
	grid $pwmPath.hide -row 0 -column 1 -sticky ne
	grid $pwmPath.channelSelect -row 1 -column 0 -sticky we -columnspan 2
	grid $pwmPath.ch1 -row 2 -column 0 -columnspan 2

}

# Adjust Waveform Generator Frequency
#--------------------------------------------
# This procedure is a service routine for the waveform generator frequency
# slider control.  It converts the slider position into a frequency, calls the
# procedure to update the hardware, and updates the frequency display.
proc pwm::adjustFrequency {channelNum sliderArg} {
	variable sliderRange
	
	if {$channelNum==1} {
		set sliderMode $pwm::sliderMode1
		set minFrequency $pwm::minFrequency1
		set maxFrequency $pwm::maxFrequency1
	} else {
		set sliderMode $pwm::sliderMode2
		set minFrequency $pwm::minFrequency2
		set maxFrequency $pwm::maxFrequency2
	}
	
	if {$sliderMode == "log"} {
		#Logarithmic interpretation of slider position
		set logMin [expr {log10($minFrequency)}]
		set logMax [expr {log10($maxFrequency)}]
		set b $logMin
		set m [expr {($logMax-$logMin)/($sliderRange-1)}]
		set y [expr {$m*($sliderArg-1)+$b}]
		set frequency [expr {pow(10,$y)}]
		
	} else {
		#Linear interpretation of slider position
		set b $minFrequency
		set m [expr {($maxFrequency-$minFrequency)/($sliderRange-1)}]
		set y [expr {$m*($sliderArg-1)+$b}]
		set frequency $y
	}
	
	#Round to the nearest tenth of a hertz
	set pwmFrequency [format "%.1f" $frequency]
	
	#Update the hardware with the new frequency
	pwm::sendFrequency $pwmFrequency $channelNum
	
	#Update the frequency display
	set pwm::frequencyDisplay$channelNum "$pwmFrequency Hz"
	
}

# Send Frequency Setting to Hardware
#-------------------------------------------
# This procedure accepts a frequency value, converts it to a four byte number,
# and sends it to the hardware.
proc pwm::sendFrequency {freq channelNum} {
	variable ddsResolution
	
	#Calculate the phase integer
	set freqOutput [expr {round($freq/$ddsResolution)}]

	#Break the phase integer into a four byte number
	set byte3 [expr {round(floor($freqOutput/pow(2,24)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,24))}]
	
	set byte2 [expr {round(floor($freqOutput/pow(2,16)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,16))}]
	
	set byte1 [expr {round(floor($freqOutput/pow(2,8)))}]
	set freqOutput [expr {$freqOutput%round(pow(2,8))}]
	
	set byte0 $freqOutput
	
	if {$channelNum == 1} {
		sendCommand "G 1 $byte3 $byte2 $byte1 $byte0"
	} elseif {$channelNum == 2} {
		sendCommand "G 2 $byte3 $byte2 $byte1 $byte0"
	}
	
}

proc pwm::adjustDuty {channelNum sliderArg} {
	
	if {$channelNum == 1} {
		set duty $pwm::duty1
	} else {
		set duty $pwm::duty2
	}
	set duty [expr round((100-$duty)*255/100.0)]
	
	if {$channelNum == 1} {
		sendCommand "P 1 $duty"
	} elseif {$channelNum == 2} {
		sendCommand "P 2 $duty"
	}
}

# Manually Set Frequency
#---------------------------
# This procedure is called when the user wants to manually set the waveform generator
# output frequency.  It presents the user with a dialog box where they can enter
# the desired output frequency.
proc pwm::setFrequency {channelNum} {
	variable minFrequencyLimit
	variable maxFrequencyLimit
	variable frequencyDisplay1
	variable frequencyDisplay2
	
	#Dialog box for user to enter the new frequency
	set newFreq [Dialog_Prompt newF "New Frequency:"]
	
	if {$newFreq == ""} {return}
	
	#Make sure that we got a valid frequency setting
	if { [string is double -strict $newFreq] } {
		if { $newFreq >= $minFrequencyLimit && $newFreq <= $maxFrequencyLimit} {
			sendFrequency $newFreq $channelNum
			set frequencyDisplay [format "%.1f" $newFreq]
			set frequencyDisplay "$frequencyDisplay Hz"
			if {$channelNum == 1} {
				set frequencyDisplay1 $frequencyDisplay
			} else {
				set frequencyDisplay2 $frequencyDisplay
			}
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency out of range"	\
			-type ok			\
			-icon warning
		}
	} else {
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $minFrequencyLimit and $maxFrequencyLimit"	\
			-type ok			\
			-icon warning
		return
	}
	

}

#Set Maximum Frequency
#---------------
#This procedure prompts the user for a new max frequency value.
#The frequency supplied by the user is checked to ensure that
#it is a valid number and a valid frequency setting.
proc pwm::setMaxFrequency {channelNum} {
	
	set newMax [Dialog_Prompt setMax "New Maximum Frequency:"]
	
	if {$newMax == ""} { return }
	
	if {$channelNum == 1} {
		set minFrequency $pwm::minFrequency1
	} else {
		set minFrequency $pwm::minFrequency2
	}
	
	if { [string is double -strict $newMax]} {
		if {$newMax > $minFrequency && $newMax <= $pwm::maxFrequencyLimit} {
			set pwm::maxFrequency$channelNum [format "%.1f" $newMax]
			set pwmPath [getPWMPath]
			pwm::adjustFrequency $channelNum [$pwmPath.ch$channelNum.freq$channelNum.freqSlider get] 
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Invalid Frequency.\nMax frequency is $pwm::maxFrequencyLimit\nMin frequency is $minFrequency"	\
			-type ok			\
			-icon warning
		}
	} else {
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $pwm::minFrequencyLimit and $pwm::maxFrequencyLimit."	\
			-type ok			\
			-icon warning
		return
	}
}

#Set Minimum Frequency
#---------------
#This procedure prompts the user for a new min frequency value.
#The frequency supplied by the user is checked to ensure that
#it is a valid number and a valid frequency setting.
proc pwm::setMinFrequency {channelNum} {

	set newMin [Dialog_Prompt setMin "New Minimum Frequency:"]
	
	if {$newMin == ""} {return}
	
	if {$channelNum == 1} {
		set maxFrequency $pwm::maxFrequency1
	} else {
		set maxFrequency $pwm::maxFrequency2
	}
	
	if { [string is double -strict $newMin] } {
		if { $newMin < $maxFrequency && $newMin >= $pwm::minFrequencyLimit} {
			set pwm::minFrequency$channelNum [format "%.1f" $newMin]
			set pwmPath [getPWMPath]
			pwm::adjustFrequency $channelNum [$pwmPath.ch$channelNum.freq$channelNum.freqSlider get]
		} else {
			tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Invalid Frequency.\nMin frequency is $pwm::minFrequencyLimit\nMax frequency is $maxFrequency"	\
			-type ok			\
			-icon warning
		}
	} else {
		tk_messageBox	\
			-title "Invalid Frequency"	\
			-default ok		\
			-message "Frequency must be a number\nbetween $pwm::minFrequencyLimit and $pwm::maxFrequencyLimit."	\
			-type ok			\
			-icon warning
		return
	}
}


proc pwm::pwmOutput {channelNum} {

	set pwmPath [getPWMPath]
	
	if {$channelNum == 1} {
		set enabled $pwm::outputEnabled1
	} else {
		set enabled $pwm::outputEnabled2
	}

	if {$enabled} {
		#Make sure all the controls are active
		$pwmPath.ch$channelNum.freq$channelNum.freqSlider configure -state normal
		$pwmPath.ch$channelNum.duty$channelNum.dutySlider configure -state normal
		#Enable the function generator output
		if {$channelNum == 1} {
			sendCommand "1"
		} elseif {$channelNum == 2} {
			sendCommand "2"
		}
	} else {
		#Disable the controls
		$pwmPath.ch$channelNum.freq$channelNum.freqSlider configure -state disabled
		$pwmPath.ch$channelNum.duty$channelNum.dutySlider configure -state disabled
		#Disable the function generator output
		if {$channelNum == 1} {
			sendCommand "!"
		} elseif {$channelNum == 2} {
			sendCommand "@"
		}
	}

}

proc pwm::selectPWM {} {

	set pwmPath [pwm::getPWMPath]

	switch $pwm::pwmChannel {
		"Pulse Channel 1" {
			grid remove $pwmPath.ch2
			grid $pwmPath.ch1 -row 2 -column 0 -columnspan 2
		} "Pulse Channel 2" {
			grid remove $pwmPath.ch1
			grid $pwmPath.ch2 -row 2 -column 0 -columnspan 2
		}
	}

}
