
package provide trigger 1.0
package require Img

namespace eval trigger {

#---=== Trigger Global Variables ===---
set waveTriggerMode normal
set waveOutputMode continuous
set patternTriggerMode normal
set triggerSource manual
set triggerCycles 1
set onCycles 1
set offCycles 2
set repeatCycles 3
set counterThreshold 8192

#---=== Export Public Procedures ===---
namespace export buildTrigger
namespace export setTriggerPath
namespace export getTriggerPath
}

#---=== Procedures ===---
proc trigger::setTriggerPath {triggerPath} {
	variable trigger
	
	set trigger(path) $triggerPath
}

proc trigger::getTriggerPath {} {
	variable trigger
	
	return $trigger(path)
}


proc trigger::buildTrigger {} {


	set triggerPath [trigger::getTriggerPath]
	
	#Frame for Trigger Controls
	frame $triggerPath	\
		-relief raised	\
		-borderwidth 2
		
	label $triggerPath.label	\
		-text "Trigger Controls"	\
		-font {-weight bold -size -16}
		
	button $triggerPath.hide	\
		-text "X"	\
		-font {-weight bold -size -12}	\
		-relief flat	\
		-command {set triggerControlEnabled 0;remapGUI}
		
	#Frame for waveform generator trigger mode controls
	frame $triggerPath.waveMode	\
		-relief groove	\
		-borderwidth 1
	
	label $triggerPath.waveMode.title	\
		-text "Waveform Generator\nTrigger Mode:"	\
		-font {-weight bold -size -12}
	
	radiobutton $triggerPath.waveMode.normal	\
		-text "Free-Running"	\
		-value normal	\
		-variable trigger::waveTriggerMode	\
		-command trigger::selectWaveTriggerMode
	
	radiobutton $triggerPath.waveMode.triggered	\
		-text "Triggered"	\
		-value triggered	\
		-variable trigger::waveTriggerMode	\
		-command trigger::selectWaveTriggerMode
	
	grid $triggerPath.waveMode.title -row 0 -sticky w
	grid $triggerPath.waveMode.normal -row 1 -sticky w
	grid $triggerPath.waveMode.triggered -row 2 -sticky w
	
	#Frame for waveform generator output mode controls
	frame $triggerPath.outMode	\
		-relief groove	\
		-borderwidth 1
		
	label $triggerPath.outMode.title	\
		-text "Waveform Generator\nOutput Mode:"	\
		-font {-weight bold -size -12}
		
	radiobutton $triggerPath.outMode.normal	\
		-text "Continuous"	\
		-value continuous	\
		-variable trigger::waveOutputMode	\
		-command trigger::selectWaveOutputMode
		
	radiobutton $triggerPath.outMode.toneBurst	\
		-text "Tone Burst"	\
		-value toneBurst		\
		-variable trigger::waveOutputMode	\
		-command trigger::selectWaveOutputMode
		
	grid $triggerPath.outMode.title -row 0 -sticky w
	grid $triggerPath.outMode.normal -row 1 -sticky w
	grid $triggerPath.outMode.toneBurst -row 2 -sticky w
	
	#Frame for pattern generator trigger mode controls
	frame $triggerPath.patternMode	\
		-relief groove	\
		-borderwidth 1
	
	label $triggerPath.patternMode.title	\
		-text "Pattern Generator\nTrigger Mode:"	\
		-font {-weight bold -size -12}
	
	radiobutton $triggerPath.patternMode.normal	\
		-text "Normal"	\
		-value normal	\
		-variable trigger::patternTriggerMode	\
		-command trigger::selectPatternTriggerMode
	
	radiobutton $triggerPath.patternMode.triggered	\
		-text "Triggered"	\
		-value triggered	\
		-variable trigger::patternTriggerMode	\
		-command trigger::selectPatternTriggerMode
	
	grid $triggerPath.patternMode.title -row 0 -sticky w
	grid $triggerPath.patternMode.normal -row 1 -sticky w
	grid $triggerPath.patternMode.triggered -row 2 -sticky w
	
	#Frame for trigger source selectors
	frame $triggerPath.source	\
		-relief groove	\
		-borderwidth 1
	
	label $triggerPath.source.title	\
		-text "Trigger Source:"	\
		-font {-weight bold -size -12}
		
	radiobutton $triggerPath.source.external	\
		-text "External"	\
		-value external	\
		-variable trigger::triggerSource	\
		-command trigger::selectTriggerSource
		
	radiobutton $triggerPath.source.manual	\
		-text "Manual"	\
		-value manual	\
		-variable trigger::triggerSource	\
		-command trigger::selectTriggerSource
		
	radiobutton $triggerPath.source.counter	\
		-text "Counter Threshold:"	\
		-value counter	\
		-variable trigger::triggerSource	\
		-command trigger::selectTriggerSource
		
	button $triggerPath.source.counterThreshold	\
		-textvariable trigger::counterThreshold	\
		-width 12	\
		-command trigger::setCounterThreshold
		
		
	radiobutton $triggerPath.source.counterIn	\
		-text "Counter Input"	\
		-value counterIn	\
		-variable trigger::triggerSource	\
		-command trigger::selectTriggerSource
	
	grid $triggerPath.source.title -row 0 -sticky w
	grid $triggerPath.source.external -row 1 -sticky w 
	grid $triggerPath.source.manual -row 2 -sticky w
	grid $triggerPath.source.counterIn -row 3 -sticky w
	grid $triggerPath.source.counter -row 4 -sticky w
	grid $triggerPath.source.counterThreshold -row 5 -sticky we
		
	frame $triggerPath.cycles	\
		-relief groove	\
		-borderwidth 1
		
	label $triggerPath.cycles.title	\
		-text "Trigger Cycles"	\
		-font {-weight bold -size -12}
	
	label $triggerPath.cycles.onLabel	\
		-text "On:"	\
		-width 8
	
	button $triggerPath.cycles.onCycles	\
		-textvariable trigger::onCycles	\
		-command trigger::modifyOnCycles	\
		-width 7
		
	label $triggerPath.cycles.offLabel	\
		-text "Off:"	\
		-width 8
		
	button $triggerPath.cycles.offCycles	\
		-textvariable trigger::offCycles	\
		-command trigger::modifyOffCycles		\
		-width 7
		
	label $triggerPath.cycles.repeatLabel	\
		-text "Repeat:"	\
		-width 8
		
	button $triggerPath.cycles.repeatCycles	\
		-textvariable trigger::repeatCycles	\
		-command trigger::modifyRepeatCycles	\
		-width 7
		
	grid $triggerPath.cycles.title -row 0 -columnspan 2
	grid $triggerPath.cycles.onLabel -row 1 -column 0
	grid $triggerPath.cycles.onCycles -row 1 -column 1
	grid $triggerPath.cycles.offLabel -row 2 -column 0
	grid $triggerPath.cycles.offCycles -row 2 -column 1
	grid $triggerPath.cycles.repeatLabel -row 3 -column 0
	grid $triggerPath.cycles.repeatCycles -row 3 -column 1
	
	button $triggerPath.manualTrigger	\
		-text "Manual Trigger"	\
		-command {sendCommand T}
		
	grid $triggerPath.label -row 0 -column 0 -sticky w
	grid $triggerPath.hide -row 0 -column 1 -sticky ne
	grid $triggerPath.waveMode -row 1 -column 0 -sticky news -columnspan 2
	grid $triggerPath.outMode -row 2 -column 0 -sticky news -columnspan 2
	grid $triggerPath.patternMode -row 3 -column 0 -sticky news -columnspan 2
	grid $triggerPath.source -row 4 -column 0 -sticky news -columnspan 2
	grid $triggerPath.cycles -row 5 -column 0 -sticky news -columnspan 2
	grid $triggerPath.manualTrigger -row 6 -column 0 -sticky news -columnspan 2 -pady 10
	
}

proc trigger::selectWaveTriggerMode {} {
		
	if {$trigger::waveTriggerMode == "triggered"} {
		sendCommand "M 1"
		if {$trigger::waveOutputMode=="toneBurst"} {
			trigger::checkTriggeredToneBurst
		}
		trigger::updateRepeatCycles
	} else {
		sendCommand "M 0"
	}
}

proc trigger::selectWaveOutputMode {} {
	
	switch $trigger::waveOutputMode {
		"continuous" {
			sendCommand "V 0"
		} "toneBurst" {
			sendCommand "V 1"
			if {$trigger::waveTriggerMode=="triggered"} {
				trigger::checkTriggeredToneBurst
				trigger::updateRepeatCycles
			}
		}
	}
}

proc trigger::selectPatternTriggerMode {} {
	
	if {$trigger::patternTriggerMode == "triggered"} {
		sendCommand "M 3"
	} else {
		sendCommand "M 2"
	}
	
}

proc trigger::selectTriggerSource {} {
	
	switch $trigger::triggerSource {
		"external" {
			sendCommand "X 0"
		} "manual" {
			sendCommand "X 1"
		} "counter" {
			sendCommand "X 2"
		} "counterIn" {
			sendCommand "X 3"
		}
	}
	
}

proc trigger::setCounterThreshold {} {

	set newThreshold [Dialog_Prompt setThresh "New Counter Threshold:"]
	
	if {$newThreshold == ""} {return}
	
	if {[string is integer -strict $newThreshold]!=1} {
		tk_messageBox	\
			-message "Threshold must be an integer."	\
			-default ok	\
			-icon error	\
			-title "Threshold Value Error"
		puts stderr "Threshold must be an integer."
		return
	}
	
	if {($newThreshold < 0) || ($newThreshold>2147483647)} {
		tk_messageBox	\
			-message "Threshold must be between 0 and 2147483647."	\
			-default ok	\
			-icon error	\
			-title "Threshold Value Error"
		puts stderr "Threshold Value Error"
		return
	}
	
	set trigger::counterThreshold $newThreshold
	
	trigger::updateCounterThreshold
}

proc trigger::updateCounterThreshold {} {
	variable counterThreshold
	
	set newThreshold $counterThreshold
	
	set byte3 [expr {round(floor($newThreshold/pow(2,24)))}]
	set newThreshold [expr {$newThreshold%round(pow(2,24))}]
	
	set byte2 [expr {round(floor($newThreshold/pow(2,16)))}]
	set newThreshold [expr {$newThreshold%round(pow(2,16))}]

	set byte1 [expr {round(floor($newThreshold/pow(2,8)))}]
	set byte0 [expr {$newThreshold%round(pow(2,8))}]
	
	sendCommand "L $byte3 $byte2 $byte1 $byte0"
}


proc trigger::modifyOnCycles {} {

	set newCycles [Dialog_Prompt setOn "New Value:"]
	
	if {$newCycles== ""} {return}
	
	if {[string is integer -strict $newCycles]!=1} {
		tk_messageBox	\
			-message "Value must be an integer."	\
			-default ok	\
			-icon error	\
			-title "Value Error"
		puts stderr "Value must be an integer."
		return
	}
	
	if {($newCycles < 0) || ($newCycles>65535)} {
		tk_messageBox	\
			-message "Threshold must be between 0 and 65535."	\
			-default ok	\
			-icon error	\
			-title "Value Error"
		puts stderr "Value Error"
		return
	}
	
	set trigger::onCycles $newCycles
	
	trigger::updateOnCycles
	trigger::updateRepeatCycles

}

proc trigger::updateOnCycles {} {
	variable onCycles
	
	set temp [expr {65536-$onCycles}]
		
	set byte1 [expr {round(floor($temp/pow(2,8)))}]
	set byte0 [expr {$temp%round(pow(2,8))}]
	
	sendCommand "Z $byte1 $byte0"
	
}

proc trigger::modifyOffCycles {} {
	
	set newCycles [Dialog_Prompt setOn "New Value:"]
	
	if {$newCycles== ""} {return}
	
	if {[string is integer -strict $newCycles]!=1} {
		tk_messageBox	\
			-message "Value must be an integer."	\
			-default ok	\
			-icon error	\
			-title "Value Error"
		puts stderr "Value must be an integer."
		return
	}
	
	if {($newCycles < 0) || ($newCycles>65535)} {
		tk_messageBox	\
			-message "Threshold must be between 0 and 65535."	\
			-default ok	\
			-icon error	\
			-title "Value Error"
		puts stderr "Value Error"
		return
	}
	
	set trigger::offCycles $newCycles
	
	trigger::updateOffCycles
	trigger::updateRepeatCycles
	
}

proc trigger::updateOffCycles {} {
	variable offCycles
	
	set temp [expr {65536-$offCycles}]
	
	set byte1 [expr {round(floor($temp/pow(2,8)))}]
	set byte0 [expr {$temp%round(pow(2,8))}]
	
	sendCommand "z $byte1 $byte0"

}

proc trigger::modifyRepeatCycles {} {
	
	set newCycles [Dialog_Prompt setOn "New Value:"]
	
	if {$newCycles== ""} {return}
	
	if {[string is integer -strict $newCycles]!=1} {
		tk_messageBox	\
			-message "Value must be an integer."	\
			-default ok	\
			-icon error	\
			-title "Value Error"
		puts stderr "Value must be an integer."
		return
	}
	

	if {($newCycles < 0) || ($newCycles>65535)} {
		tk_messageBox	\
			-message "Value must be between 0 and 65535."	\
			-default ok	\
			-icon error	\
			-title "Value Error"
		puts stderr "Value Error"
		return
	}
	
	set trigger::repeatCycles $newCycles

	trigger::updateRepeatCycles
}

proc trigger::updateRepeatCycles {} {
	variable repeatCycles

	if {($trigger::waveTriggerMode=="triggered")&&($trigger::waveOutputMode=="toneBurst")} {
		trigger::checkTriggeredToneBurst
		set temp [expr {65536-($trigger::onCycles+$trigger::offCycles)*$repeatCycles}]
		if {$temp < 0} {set temp 0}
	} else {
		set temp [expr {65536-$repeatCycles}]
	}
	
	set byte1 [expr {round(floor($temp/pow(2,8)))}]
	set byte0 [expr {$temp%round(pow(2,8))}]
	
	sendCommand "Y $byte1 $byte0"
	

}

proc trigger::checkTriggeredToneBurst {} {

	set temp [expr {($trigger::onCycles+$trigger::offCycles)*$trigger::repeatCycles}]

	if {[expr {65535.0/$temp}] < 1} {
		tk_messageBox	\
			-message "The total number of cycles: (on + off) x repeat\nmust be less than 65535."	\
			-default ok	\
			-icon error	\
			-title "Value Error"
		set trigger::repeatCycles 1
	}

}
