namespace eval waveEdit {

set waveform {}
set fileName ""
set modified 0
set waveformLength 1024

set gridHeight 20
set gridWidth 35

set displayHeight [expr {$waveEdit::gridHeight*17}]
set displayWidth [expr {$waveEdit::gridWidth*2*10}]

set displayExtent [expr {$waveEdit::gridWidth*1025}]

set scrollList {0 0}
lappend scrollList $displayExtent
lappend scrollList $displayHeight

set waveImages(new) [image create photo -file "./Images/NewButton.gif"]
set waveImages(save) [image create photo -file "./Images/SaveButton.gif"]
set waveImages(open) [image create photo -file "./Images/OpenButton.gif"]

}

proc waveEdit::showWaveEdit {} {

	if {![winfo exists .waveEdit]} {
	
		toplevel .waveEdit
		wm title .waveEdit "Waveform Editor"
		wm protocol .waveEdit WM_DELETE_WINDOW {waveEdit::closeWaveEdit}

		#Create a frame for the standard menubar
		frame .waveEdit.menubar -relief raised -borderwidth 1

		#Create the file menu
		menubutton .waveEdit.menubar.file	\
			-text "File"	\
			-menu .waveEdit.menubar.file.fileMenu	\
			-width 6
		menu .waveEdit.menubar.file.fileMenu -tearoff 0
		.waveEdit.menubar.file.fileMenu add command	\
			-label "New Waveform"	\
			-command waveEdit::newWaveform
		.waveEdit.menubar.file.fileMenu add separator
		.waveEdit.menubar.file.fileMenu add command	\
			-label "Save Waveform"	\
			-command waveEdit::saveWaveform
		.waveEdit.menubar.file.fileMenu add command	\
			-label "Save Waveform as..."	\
			-command waveEdit::saveWaveformAs
		.waveEdit.menubar.file.fileMenu add separator
		.waveEdit.menubar.file.fileMenu add command	\
			-label "Load Waveform..."	\
			-command waveEdit::loadWaveform
		.waveEdit.menubar.file.fileMenu add separator
		.waveEdit.menubar.file.fileMenu add command	\
			-label "Exit"	\
			-command {waveEdit::closeWaveEdit}
			
		grid .waveEdit.menubar.file -row 0 -column 0 -sticky w

		#Create a frame for the button bar
		frame .waveEdit.buttonBar -relief raised -borderwidth 1

		button .waveEdit.buttonBar.new	\
			-image $waveEdit::waveImages(new)	\
			-command waveEdit::newWaveform

		button .waveEdit.buttonBar.open	\
			-image $waveEdit::waveImages(open)	\
			-command waveEdit::loadWaveform

		button .waveEdit.buttonBar.save	\
			-image $waveEdit::waveImages(save)	\
			-command waveEdit::saveWaveform
			
		grid .waveEdit.buttonBar.new -row 0 -column 0 -sticky w
		grid .waveEdit.buttonBar.open -row 0 -column 1 -sticky w
		grid .waveEdit.buttonBar.save -row 0 -column 2 -sticky w

		canvas .waveEdit.display	\
			-height $waveEdit::displayHeight	\
			-width $waveEdit::displayWidth	\
			-scrollregion $waveEdit::scrollList	\
			-background white	\
			-xscrollcommand ".waveEdit.scroller set"	\
			-borderwidth 0

		scrollbar .waveEdit.scroller	\
			-orient horizontal	\
			-command ".waveEdit.display xview"

		grid .waveEdit.menubar -row 0 -column 0 -sticky nw -columnspan 3
		grid .waveEdit.buttonBar -row 1 -column 0 -sticky nw -columnspan 3
		grid .waveEdit.display -row 2 -column 0 -sticky news
		grid .waveEdit.scroller -row 3 -column 0 -sticky we
		
		bind .waveEdit.display <ButtonPress-1> {%W scan mark %x %y;set waveEdit::clickMotion 0}
		bind .waveEdit.display <B1-Motion> {%W scan dragto %x %y 1;set waveEdit::clickMotion 1}
		bind .waveEdit.display <ButtonRelease-1> {waveEdit::updateWaveform %x %y}

		#waveEdit::initializeWaveform
	} else {
		wm deiconify .waveEdit
		raise .waveEdit
		focus .waveEdit
	}
}


proc waveEdit::drawGrid {} {

	#Make sure the canvas is the right size for the number of samples
	set waveEdit::displayExtent [expr {$waveEdit::gridWidth*($waveEdit::waveformLength+1)}]
	set waveEdit::scrollList {0 0}
	lappend waveEdit::scrollList $waveEdit::displayExtent
	lappend waveEdit::scrollList $waveEdit::displayHeight
	
	.waveEdit.display	configure -scrollregion $waveEdit::scrollList

	#Delete any old graphics
	.waveEdit.display delete waveDisplay

	#Create text labels for sample numbers
	set y [expr {$waveEdit::gridHeight/2}]
	for {set i 0} {$i < $waveEdit::waveformLength} {incr i} {
		set x [expr {$waveEdit::gridWidth*($i+1)+$waveEdit::gridWidth/2}]
		.waveEdit.display create text \
			$x $y	\
			-text $i	\
			-tag waveDisplay
	}
	
	#Draw the grid
	set y1 0
	set y2 $waveEdit::displayHeight
	for {set i 0} {$i < [expr {$waveEdit::waveformLength+1}]} {incr i} {
		set x [expr {$i*$waveEdit::gridWidth}]
		.waveEdit.display create line	\
		$x $y1 $x $y2	\
		-fill grey		\
		-tag grid	\
		-tag waveDisplay
	}
	
	#Create bit number labels
	set x [expr {$waveEdit::gridWidth/2}]
	for {set i 0} {$i <8} {incr i} {
		set y [expr {$waveEdit::gridHeight*($i*2+1)+$waveEdit::gridHeight/2}]
		.waveEdit.display create text	\
			$x $y	\
			-text "Bit $i"	\
			-tag waveDisplay
	}
	
	#Draw the invidual bit waveforms
	for {set i 0} {$i <8} {incr i} {
		waveEdit::drawWaveform $i
	}

}

proc waveEdit::updateWaveform {x y} {
	variable clickMotion
	
	if {!$clickMotion} {
		set x [.waveEdit.display canvasx $x]
		set y [.waveEdit.display canvasy $y]
		if {($x > $waveEdit::gridWidth)&&($x < $waveEdit::displayExtent)} {
			set sampleNum [expr {round(floor(($x-$waveEdit::gridWidth)/$waveEdit::gridWidth))}]
		} else {
			return
		}
		if {($y > [expr {$waveEdit::gridHeight/2}])&&($y < [expr {$waveEdit::gridHeight*(7*2+1)+$waveEdit::gridHeight-2}])} {
			set bitNum [expr {round(floor(($y-$waveEdit::gridHeight/2)/($waveEdit::gridHeight*2)))}]
		} else {
			return
		}
		
		waveEdit::togglebit $sampleNum $bitNum
		waveEdit::drawWaveform $bitNum
		
		set waveEdit::modified 1
	}
	
	
	
}

proc waveEdit::initializeWaveform {} {

	set getLength [tk_dialog .waveEdit.getWaveLength "New Waveform" "Select Waveform Length:"	\
		questhead 8 "4" "8" "16" "32" "64" "128" "256" "512" "1024"]
		
	set waveEdit::waveformLength [expr {round(pow(2,$getLength)*4)}]
		
	puts "User selected $waveEdit::waveformLength"

	set waveEdit::waveform {}

	for {set i 0} {$i < $waveEdit::waveformLength} {incr i} {
		lappend waveEdit::waveform 0
	}
	
	set waveEdit::fileName ""
	set waveEdit::modified 0
	
	waveEdit::drawGrid
	
	raise .waveEdit

}

proc waveEdit::drawWaveform {bitNum} {

	.waveEdit.display delete bitWave$bitNum

	set x $waveEdit::gridWidth
	set yOffset [expr {$waveEdit::gridHeight*($bitNum*2+1)}]
	set coordList {}
	for {set i 0} {$i < $waveEdit::waveformLength} {incr i} {
	
		set bitValue [waveEdit::getBit $i $bitNum]
	
		lappend coordList $x
		if {$bitValue} {
			lappend coordList [expr {$yOffset+2}]
			set x [expr {$x+$waveEdit::gridWidth}]
			lappend coordList $x
			lappend coordList [expr {$yOffset+2}]
		} else {
			lappend coordList [expr {$yOffset+$waveEdit::gridHeight-2}]
			set x [expr {$x+$waveEdit::gridWidth}]
			lappend coordList $x
			lappend coordList [expr {$yOffset+$waveEdit::gridHeight-2}]
		}
				
	}
	
	.waveEdit.display create line	\
		$coordList	\
		-fill black	\
		-tag bitWave$bitNum

}

proc waveEdit::getBit {sample bitNum} {

	set value [lindex $waveEdit::waveform $sample]
	
	set temp ""
	while {$value>0} {
		set temp [expr {$value%2}]$temp
		set value [expr {$value/2}]
	}
	if {$temp==""} {
		set temp {0 0 0 0 0 0 0 0}
	} else {
		set temp [split $temp ""]
		#puts [llength $temp]
		if {[llength $temp] < 8} {
			set i [expr {8-[llength $temp]}]
			#puts "i is $i"
			while {$i > 0} {
				set temp [linsert $temp 0 0]
				set i [expr {$i - 1}]
			}
		}
	}
	

	return [lindex $temp [expr {7-$bitNum}]]

}

proc waveEdit::setBit {sample bitNum} {

	set value [lindex $waveEdit::waveform $sample]
	
	if {[waveEdit::getBit $sample $bitNum]} {
		return
	} else {
		set value [expr {round($value+pow(2,$bitNum))}]
		set waveEdit::waveform [lreplace $waveEdit::waveform $sample $sample $value] 
	}
}

proc waveEdit::clearBit {sample bitNum} {

	set value [lindex $waveEdit::waveform $sample]
	
	if {![waveEdit::getBit $sample $bitNum]} {
		return
	} else {
		set value [expr {round($value-pow(2,$bitNum))}]
		set waveEdit::waveform [lreplace $waveEdit::waveform $sample $sample $value] 
	}

}

proc waveEdit::togglebit {sample bitNum} {

	set bitValue [waveEdit::getBit $sample $bitNum]
	
	if {$bitValue} {
		waveEdit::clearBit $sample $bitNum
	} else {
		waveEdit::setBit $sample $bitNum
	}
}

proc waveEdit::getSaveFileName {} {

	set types {
			{{Text Files} {.txt}}
		}
	
	set dataFile [tk_getSaveFile	\
		-filetypes $types	\
		-initialfile "Pattern.txt"	\
		-defaultextension .txt]
		
	return $dataFile

}

proc waveEdit::writeWaveform {fileName} {
	
	if {[catch {open $fileName w} fileId]} {
		tk_messageBox	\
			-message "Unable to save file.\nCould not open file for writing."	\
			-type ok
		return
	}
	
	puts $fileId $waveEdit::waveformLength
	
	foreach sample $waveEdit::waveform {
		puts $fileId $sample
	}
	
	close $fileId
	
	set waveEdit::fileName $fileName
	set waveEdit::modified 0
}

proc waveEdit::saveWaveform {} {
	
	if {$waveEdit::fileName == ""} {
		set dataFile [waveEdit::getSaveFileName]
	} else {
		set dataFile $waveEdit::fileName
	}
	
	if {$dataFile == ""} {return}
	
	waveEdit::writeWaveform $dataFile
	
}

proc waveEdit::saveWaveformAs {} {

	set dataFile [waveEdit::getSaveFileName]
	
	if {$dataFile == ""} {return}
	
	waveEdit::writeWaveform $dataFile

}

proc waveEdit::newWaveform {} {

	if {$waveEdit::modified} {
		set answer [tk_messageBox	\
			-message "Waveform has been modified.\nWould you like to save the waveform?"	\
			-type yesno	\
			-default yes]
		
		if {$answer == "yes"} {
			waveEdit::saveWaveform
			if {$waveEdit::modified} {
				return
			} else {
				waveEdit::initializeWaveform
			}
		} else {
			waveEdit::initializeWaveform
		}
	} else {
		waveEdit::initializeWaveform
	}


}

proc waveEdit::loadWaveform {} {

	set types {
			{{Text Files} {.txt}}
		}
	
	set dataFile [tk_getOpenFile	\
		-filetypes $types	\
		-initialfile "Pattern.txt"	\
		-defaultextension .txt]
		
	if {$dataFile == ""} {
		return
	}
	
	if {[catch {open $dataFile r} fileId]} {
		tk_messageBox	\
			-message "Unable to open file.\nCould not open file for reading."	\
			-type ok
		return
	}
	
	set startTime [clock milliseconds]
	
	set i 0
	set newWaveform {}
	gets $fileId recordLength
	
	while {[gets $fileId line] >= 0} {
	
		incr i
		#Make sure that we don't exceed the waveform length defined in the first line of the file
		if {$i > $recordLength} {
			tk_messageBox	\
			-message "Too many samples/lines in file!\nTotal number of samples should be $recordLength."	\
			-type ok	\
			-default ok
			return
		}
		
		#Make sure that the sample is valid
		if {[string is integer -strict $line]} {
			if {($line >= 0) && ($line < 255)} {
				lappend newWaveform $line
			} else {
				tk_messageBox	\
					-message "Sample on line $i is invalid.\nValid sample values are 0 to 255."	\
					-type ok	\
					-default ok
				return
			}
		} else {
			tk_messageBox	\
				-message "Invalid sample value on line $i.\nSamples must be integers between 0 and 255"	\
				-type ok	\
				-default ok
			return
		}
	
	}

	if {$i < $recordLength} {
		tk_messageBox	\
			-message "Not enough samples/lines in file!\nTotal number of samples should be $recordLength.\n"	\
			-type ok	\
			-default ok
			return
	}

	set readTime [clock milliseconds]

	set waveEdit::waveform $newWaveform
	set waveEdit::waveformLength $recordLength

	set waveEdit::fileName $dataFile
	set waveEdit::modified 0
	
	set setupTime [clock milliseconds]
	
	waveEdit::drawGrid
	
	set endTime [clock milliseconds]
	
	puts "Total time: [expr {$endTime-$startTime}]"
	puts "Draw time: [expr {$endTime-$setupTime}]"

}

proc waveEdit::closeWaveEdit {} {
	
	if {$waveEdit::modified} {
		set answer [tk_messageBox	\
			-message "Waveform has been modified.\nWould you like to save the waveform?"	\
			-type yesno	\
			-default yes]
		
		if {$answer == "yes"} {
			waveEdit::saveWaveform
			if {$waveEdit::modified} {
				return
			} else {
				destroy .waveEdit
			}
		} else {
			destroy .waveEdit
		}
	} else {
		destroy .waveEdit
	}

	
}
